module MultiSet 

( MultiSet -- abstrakt
, toList, make
, empty, unit, assocs
, elementOf, multiplicity
, plus, minus
)

where

import FiniteMap
import ToDoc hiding ( empty )

data MultiSet a = MultiSet (FiniteMap a Int)

toList :: Ord a => MultiSet a -> [a]
toList (MultiSet fm) = do
    ( x, k ) <- fmToList fm
    replicate k x

assocs :: Ord a => MultiSet a -> [(a, Int)]
assocs (MultiSet fm) = fmToList fm

make :: Ord a => [a] -> MultiSet a
make xs = MultiSet $ addListToFM_C (+) emptyFM $ do
      x <- xs
      return (x, 1)

empty :: Ord a => MultiSet a
empty = make []

unit :: Ord a => a -> MultiSet a
unit x = make [ x ]

elementOf :: Ord a => a -> MultiSet a -> Bool
elementOf x ms = 0 < multiplicity ms x

multiplicity :: Ord a => MultiSet a -> a -> Int
multiplicity (MultiSet fm) x = case lookupFM fm x of
    Just c -> c; Nothing -> 0

plus :: Ord a => MultiSet a -> MultiSet a -> MultiSet a
plus (MultiSet fm1) (MultiSet fm2) = 
    MultiSet $ addListToFM_C (+) fm1 $ fmToList fm2

minus :: Ord a => MultiSet a -> MultiSet a -> MultiSet a
minus (MultiSet fm1) (MultiSet fm2) = 
    MultiSet $ purge
	     $ addListToFM_C (+) fm1 $ do
	          ( x, c ) <- fmToList fm2
		  return ( x, negate c )

purge :: Ord a => FiniteMap a Int -> FiniteMap a Int
purge = filterFM ( \ x c -> c > 0 )

instance (Ord a, ToDoc [a]) => ToDoc (MultiSet a) where
    toDoc ms = text "MultiSet.make" <+> toDoc ( toList ms )

instance (Ord a, ToDoc [a]) => Show (MultiSet a) where
    show = render . toDoc

instance Ord a => Eq (MultiSet a) where -- lexikografisch
    x == y = toList x == toList y

instance Ord a => Ord (MultiSet a) where -- lexikografisch
    x `compare` y = toList x `compare` toList y

