module Takebreak where

import Next

import qualified Set as S
import OrdSet
import qualified MultiSet as M

import List (inits, tails)
import Monad (guard)
import ToDoc


data TB = TB { rules :: [ S.Set Int ]
	     , heaps :: M.MultiSet Int
	     }
     deriving ( Eq, Ord )

------------------------------------------------------------------------------

instance ToDoc TB where
    toDoc tb = braces $ fsep $ punctuate comma $ map toDoc 
	     $ M.toList $ heaps tb

instance Show TB where
    show = render . toDoc

instance Next TB where
    next tb = do
        ( x, c ) <- M.assocs $ heaps tb
	( d, m ) <- zip [0..] $ rules tb
	s <- S.setToList m
	xs <- splits s (x - d)
	return $ tb { heaps = (heaps tb `M.minus` M.unit x) `M.plus` xs }

------------------------------------------------------------------------------
	
splits :: Int -> Int -> [ M.MultiSet Int ]
-- alle Zerlegungen von x in genau s positive summanden
splits = splitsFrom 1 

splitsFrom :: Int -> Int -> Int -> [ M.MultiSet Int ]
-- alle Summanden >= f
splitsFrom f 0 x = do
    guard $ 0 == x
    return $ M.empty
splitsFrom f s x = do
    guard $ f * s <= x
    h <- [ f .. x `div` s ]
    hs <- splitsFrom h (s - 1) (x - h)
    return $ M.unit h `M.plus` hs
    

