-- $Id: Options.hs,v 1.1.1.1 2002/05/24 10:46:45 challenger Exp $

module Options where

-- $Log: Options.hs,v $
-- Revision 1.1.1.1  2002/05/24 10:46:45  challenger
-- start
--
-- Revision 1.4  2002/01/18 13:41:31  mrahn
-- - ultrasimples per-User-Slot-Locking eingebaut
--
-- Revision 1.3  2001/12/02 18:46:18  mrahn
-- typos raus
--
-- Revision 1.2  2001/12/02 18:37:37  mrahn
-- - verbesserungen an vorkompilierten gradern
-- - alle aufgaben aus ws01 mit vorkompilierten gradern umgesetzt
--
-- Revision 1.1  2001/11/22 10:41:16  mrahn
-- - Kommandozeile verarbeiten
--

import GetOpt
import Settings_Type
import Defaults

data Flag = Version
	  | ShowSettings

	  | Sourcedir String
          | Workdir   String
	  | Donedir   String
	  | Bindir    String
	  | Configdir String
	  | Logdir    String
	  | Crapdir   String
	  | Queuedir  String
	  | Lockdir   String

	  | Logfile    String
	  | Resultfile String
	  | Crapfile   String

	  | Crapfile_ext   String

	  | Dispatch_file  String
	  | User_files     String
	  | Signature_file String

	  | Verboten       String

	  | Max_mail_size              Integer
	  | Mail_to_large_message_file String

	  | Max_length_of_showed_input  Integer
	  | Input_to_large_message_file String

          | Slot_not_free_message_file String

          | Max_mem_for_grader String

	  | Runhugs String
	  | Ghc     String
	  | Rmail   String
	    deriving (Eq)

-- the options-table for getopt including default-values for usage
options :: [OptDescr Flag]
options = map optInfoToOpt (optInfo $ adjust default_settings)
 where optInfoToOpt (s,l,a,i,v) = Option s [l] a (i ++ add v)
       add "" = ""; add xs = " (" ++ xs ++ ")"

optInfo :: Settings -> [([Char],String,ArgDescr Flag,String,String)]
optInfo set = 
 [ ( "V" , "version"       , NoArg  Version      , "show version and exit"         , "" )
 , ( "t" , "show-settings" , NoArg  ShowSettings , "show actual settings and exit" , "" )

 , ( "S" , "source-dir" , ReqArg Sourcedir "DIR" , "root directory"    , sourcedir set )
 , ( "W" , "work-dir"   , ReqArg Workdir   "DIR" , "working directory" , workdir set   )
 , ( "D" , "done-dir"   , ReqArg Donedir   "DIR" , "done directory"    , donedir set   )
 , ( "B" , "bin-dir"    , ReqArg Bindir    "DIR" , "bin directory"     , bindir set    )
 , ( "C" , "config-dir" , ReqArg Configdir "DIR" , "config directory"  , configdir set )
 , ( "L" , "log-dir"    , ReqArg Logdir    "DIR" , "log directory"     , logdir set    )
 , ( "P" , "crap-dir"   , ReqArg Crapdir   "DIR" , "crap directory"    , crapdir set   )
 , ( "Q" , "queue-dir"  , ReqArg Queuedir  "DIR" , "queue directory"   , queuedir set  )
 , ( "K" , "lock-dir"   , ReqArg Lockdir   "DIR" , "lock directory"    , lockdir set   )

 , ( "l", "log-file"    , ReqArg Logfile    "FILE" , "logfile"    , logfile set    )
 , ( "r", "result-file" , ReqArg Resultfile "FILE" , "resultfile" , resultfile set )
 , ( "c", "crap-file"   , ReqArg Crapfile   "FILE" , "crapfile"   , crapfile set   )

 , ( "" , "crap-file-ext" , ReqArg Crapfile_ext "EXT" , "crapfile extension" , crapfile_ext set )

 , ( "d", "dispatch-file"  , ReqArg Dispatch_file  "FILE" , "dispatchfile with exercise-definition" , dispatch_file set     )
 , ( "u", "user-file"      , ReqArg User_files     "FILE" , "userfiles with known users"            , show $ user_files set )
 , ( "s", "signature-file" , ReqArg Signature_file "FILE" , "file with email signature"             , signature_file set    )

 , ( "m", "mail-tlm-file" , ReqArg Mail_to_large_message_file "FILE" , "file with mail-to-large-message" , mail_to_large_message_file set )
 , ( "z", "max-mail-size" , ReqArg (Max_mail_size . read)     "NUM"  , "no bigger emails will send"      , show $ max_mail_size set       )

 , ( "p", "max-input-size" , ReqArg (Max_length_of_showed_input . read) "NUM"  , "no bigger input will accepted"    , show $ max_length_of_showed_input set )
 , ( "i", "input-tlm-file" , ReqArg Input_to_large_message_file         "FILE" , "file with input-to-large-message" , input_to_large_message_file set       )

 , ( "f", "slot-nfm-file" , ReqArg Slot_not_free_message_file           "FILE" , "file with slot-not-free-message" , slot_not_free_message_file set       )

 , ( "g", "max-mem-for-grader" , ReqArg Max_mem_for_grader "STRING" , "max mem for precompiled grader as RTS-M-Flag" , max_mem_for_grader set )

 , ( "y", "deny" , ReqArg Verboten "STRING" , "this words are disallowed" , show $ verboten set )

 , ( "H", "runhugs" , ReqArg Runhugs "STRING" , "runhugs command" , runhugs set )
 , ( "G", "ghc"     , ReqArg Ghc     "STRING" , "ghc command"     , ghc set     )
 , ( "R", "rmail"   , ReqArg Rmail   "STRING" , "mailer command"  , rmail set   )
 ]
