module Main (main) where

-------------------------------------------------------------------------------
-- Erzeugt Scorelisten
--
-- Parameter: 1: File mit Definitionen der Scores (Scorer.defs)
--            2: RES-File
-------------------------------------------------------------------------------
-- $Id: Scorer.hs,v 1.1.1.1 2002/05/24 10:46:45 challenger Exp $
-------------------------------------------------------------------------------
-- $Log: Scorer.hs,v $
-- Revision 1.1.1.1  2002/05/24 10:46:45  challenger
-- start
--
-- Revision 1.10  2001/12/03 10:41:53  mrahn
-- Scorer verarbeitet mehrere RES-Files
--
-- Revision 1.8  2001/11/22 13:57:40  autotool
-- - nur definierte Scores
--
-- Revision 1.5  2001/11/20 22:19:23  mrahn
-- - Scorer kann Leerzeilen und Kommentare
-- - Scorer gibt "best known" mit aus, zum Anheizen :-6
--
-- Revision 1.4  2001/11/08 12:05:32  mrahn
-- - Score of Scores, Kosmetik
--
-------------------------------------------------------------------------------

import FiniteMap
import System
import Util

scoreItems  = 10
scorePoints = [20,15,10,7,6,5,4,3,2,1]

data Info = Info
          { size     :: Int
	  , date     :: [Int]
	  , matrikel :: Int
	  }
	  deriving (Eq,Ord)

instance Show Info where
    showsPrec p i = showString $ (spaci 4 $ abs $ size i) ++ "  " 
		  ++ (spaci 8 $ matrikel i) ++ "  "
		  ++ (nulli 2 $ (date i) !! 2) ++ "."
		  ++ (nulli 2 $ (date i) !! 1) ++ "."
		  ++ (nulli 4 $ (date i) !! 0) ++ " "
		  ++ (nulli 2 $ (date i) !! 3) ++ ":"
		  ++ (nulli 2 $ (date i) !! 4) ++ ":"
		  ++ (nulli 2 $ (date i) !! 5)

spaci n = stretch n . show
nulli n = stretchWith '0' n . show

-------------------------------------------------------------------------------
-- komisch, aber ich habs nirgendwo gefunden

monthFM :: FiniteMap String Int 
monthFM    = listToFM [ ("Jan", 1),("Feb", 2),("Mar", 3),("Apr", 4)
		      , ("May", 5),("Jun", 6),("Jul", 7),("Aug", 8)
		      , ("Sep", 9),("Oct",10),("Nov",11),("Dec",12)
                      ]
monthNum m = lookupWithDefaultFM monthFM 13 m

-------------------------------------------------------------------------------
-- immer wieder schn

sort :: (Eq a,Ord a) => [a] -> [a]
sort [] = []
sort (x:xs) = sort [ x' | x' <- xs, x' <  x ] ++ [ x ] 
	      ++ sort [ x' | x' <- xs, x' >= x ]

-------------------------------------------------------------------------------
-- einlesen der Resultate

data ScoreType  = High | Low
type ScoreDefFM = FiniteMap String ScoreType
type DataFM     = FiniteMap String [Info]

readToFM :: ScoreDefFM -> [String] -> DataFM
readToFM scoreDef = foldl (plusFM_C (++)) emptyFM . map (readInfo scoreDef)

readInfo :: ScoreDefFM -> String -> DataFM
readInfo scoreDef line = 
    case lookupFM scoreDef aufg of
     Nothing -> emptyFM
     Just t -> addToFM emptyFM aufg 
	    [ Info 
	      { date     = [ read     $ field 6 date'            -- Jahr
                           , monthNum $ field 2 date'            -- Monat
                           , read     $ field 3 date'            -- Tag
                           ]
                           ++                                    -- St:Mi:Se
                           [ read x | x <- words $ map mySub (field 4 date') ]
	      , matrikel = read $ field  4 line'
	      , size     = case getScoreDef scoreDef aufg of
                            High -> -( read $ field 11 line' )  -- negativ bei
                            Low  ->    read $ field 11 line'    -- highscores
	      }
            ]
    where field n = head . drop (n-1)
          mySub x | x == ':'  = ' '
                  | otherwise = x
          line'   = dropWhile (/=")") wl
	  date'   = takeWhile (/="(") wl
          wl      = words line
          aufg    = field 6 line'

-------------------------------------------------------------------------------
-- einlesen der Definitionen

readScoreDefs :: [String] -> ScoreDefFM
readScoreDefs = foldl build emptyFM . map words
		where 
		 build f l = addToFM f (head l) (bd l)
                 bd = buildDef . head . drop 1

buildDef ::  String -> ScoreType
buildDef "high" = High
buildDef _      = Low

getScoreDef :: ScoreDefFM -> String -> ScoreType
getScoreDef f = lookupWithDefaultFM f Low

-------------------------------------------------------------------------------
-- ausgabe der scores

getScore :: [Info] -> [Info]
getScore = take scoreItems . reverse . buildScore [] []

-- sammelt nur das jeweils erste Auftreten einer Matrikelnummer raus
buildScore :: [Int] -> [Info] -> [Info] -> [Info]
buildScore mats is [] = is
buildScore mats is (x:xs) 
  | matrikel x > 1024 
    && not (matrikel x `elem` mats) = buildScore ((matrikel x):mats) (x:is) xs
  | otherwise                       = buildScore mats is xs

-------------------------------------------------------------------------------

a_heading :: ScoreDefFM -> String -> Int -> String
a_heading scoreDef aufg opt = 
    aufg ++ " " ++ case getScoreDef scoreDef aufg of
                    High -> "(high, "
		    Low  -> "(low, "
         ++ "best known: " ++ show opt ++ ")"

a_table :: ScoreDefFM -> ( String, [Info] ) -> ( String, [Info] )
a_table scoreDef (aufg,szs) = 
    let sszs = sort szs
    in ( a_heading scoreDef aufg (abs $ size $ head sszs)
       , getScore sszs
       )

main = do
    args <- getArgs

    defs <- readFile $ args !! 0
    let scoreDefFM = readScoreDefs 
		     $ filter ( (/='#') . head . dropWhile (==' ') )
		     $ filter ( or . map (/=' ') )
		     $ lines defs

    contents <- mapM readFile (drop 1 args)
    let dataFM = readToFM scoreDefFM 
                          $ filter ( '#' `elem` )
			  $ lines 
			  $ concat contents

    zeit <- zeit
    putStrLn $ heading $ "autotool -- Top Ten Scores, Stand von: " ++ zeit

    let scores = filter ((/= []) . snd) 
		 $ map (a_table scoreDefFM) 
		 $ fmToList dataFM

    let scoreOfScores = 
         concat 
	 $ map ( \ szs -> zip (map matrikel szs) (zip scorePoints $ repeat 1) )
	 $ map snd 
	 $ scores

    let scoreOfScoresFM = addListToFM_C pairAdd emptyFM scoreOfScores

    let scoreOut (aufg,szs) = unlines $ aufg : (map show $ szs)

    mapM ( putStrLn . scoreOut ) scores

    putStrLn $ unlines $
      "Top Ten of Top Tens (Punkte, Anzahl Scores)"
      :
      ( map ( \ (s,sc,m) -> (spaci 4 $ abs s) ++ (spaci 3 sc) ++ (spaci 9 m) )
      $ take scoreItems 
      $ sort 
      $ map ( \ (m,s) -> (-(fst s), snd s, m) ) 
      $ fmToList scoreOfScoresFM
      )
      ++
      [ "Dabei gibt es pro Score " ++ show scorePoints 
	++ " Punkte fr die Pltze [1.." ++ show scoreItems ++ "]" ]
