module Main (main) where

---------------------------------------------------------------------------
-- Fasst Ergebnisse aus einem Resultatfile zusammen und gibt sie
-- auf verschiedene Arten aus
--
-- Parameter: 1: Datei mit der Liste der Einschreibungen
--            2: Datei mit den Resultaten
---------------------------------------------------------------------------
-- $Id: Statistik.hs,v 1.1.1.1 2002/05/24 10:46:45 challenger Exp $
---------------------------------------------------------------------------
-- $Log: Statistik.hs,v $
-- Revision 1.1.1.1  2002/05/24 10:46:45  challenger
-- start
--
-- Revision 1.8  2002/01/18 08:48:29  autotool
-- ohne email
--
-- Revision 1.7  2002/01/18 08:24:50  autotool
-- concat in main umgeordnet
--
-- Revision 1.6  2001/12/03 10:59:45  mrahn
-- auch Statistik bitte mit mehreren RES-Files
--
-- Revision 1.5  2001/11/21 17:37:06  mrahn
-- - Util.hs hostet mehrfach benutztes Zeug
-- - Laufzeit mit in LOG
-- - Scorer/Statistik robuster, Kosmetik
--
-- Revision 1.4  2001/11/05 14:11:21  autotool
-- - Ausgabe verkrzt
--
-- Revision 1.3  2001/10/21 18:47:53  mrahn
-- - Link auf Statistik eingebaut
-- - Kosmetik am Statistik-Output
--
-- Revision 1.2  2001/10/21 17:37:01  mrahn
-- - Ausgabereihenfolge angepasst
-- - Uhrzeit der Berechnung mit ausgeben
--
---------------------------------------------------------------------------

import FiniteMap
import System
import Util


---------------------------------------------------------------------------
-- u.a. sowas steht im Resultatfile
---------------------------------------------------------------------------

data Info = Info
          { 	    matrikel :: String
	  , aufgabe  :: String
	  , resultat :: String
	  -- , email    :: String
	  }


---------------------------------------------------------------------------
-- Extrahieren aus einer Zeile: Datum am Anfang lschen und dann die
-- entsprechenden Felder lesen. Bei Formatnderungen hier ansetzen.
---------------------------------------------------------------------------

readInfo :: String -> Info
readInfo line = 
    let l' = dropWhile (/=")") $ words line
    in  Info 
        { matrikel = field 4 l'
	, aufgabe  = field 6 l'
	, resultat = field 8 l'
	-- , email    = field 2 l'
	}
    where field n = head . drop (n-1)


---------------------------------------------------------------------------
-- BaseFM wird mit User/Aufgabe als key verwendet und ist dann elt in
-- einem InfoFM mit Aufgabe/User als key.
-- Also einmal: InfoFM ( Aufgabe |-> BaseFM ( User |-> YesNo ) )
-- und einmal: InfoFM ( User |-> BaseFM ( Aufgabe |-> YesNo ) )
---------------------------------------------------------------------------

type YesNo  = (Int,Int)
type BaseFM = FiniteMap String YesNo
type InfoFM = FiniteMap String BaseFM

                    -- baue zwei FM, fr jede Darstellung einen
mkFMs :: [Info] -> (InfoFM, InfoFM)
mkFMs is = let ma = foldl (mkFM matrikel aufgabe) emptyFM is
               am = foldl (mkFM aufgabe matrikel) emptyFM is
	   in (ma,am)

mkFM :: (Info -> String) -> (Info -> String) -> InfoFM -> Info -> InfoFM
mkFM from to fm i = addToFM_C (plusFM_C pairAdd) fm (from i) $ mkBaseFM to i

mkBaseFM :: (Info -> String) -> Info -> BaseFM
mkBaseFM key i = addToFM emptyFM (key i) $ yesno i

yesno :: Info -> YesNo
yesno i | resultat i == "OK" = (1,0) -- erste Komponete OK
        | otherwise          = (0,1)


---------------------------------------------------------------------------

count :: (YesNo -> Int) -> BaseFM -> Int
count f = sum . map f . eltsFM

ok :: BaseFM -> Int
ok = count fst

no :: BaseFM -> Int
no = count snd


---------------------------------------------------------------------------
-- Listen ausgeben:
---------------------------------------------------------------------------

              -- Bezeichnung des vierten Feldes (Aufgabe/User)
showInfoFM :: String -> InfoFM -> String
showInfoFM t = concat . map f . fmToList
		where f (s,bm) = unlines $ 
				 [ " * " ++ s ++ ": " ++ overview t bm
				 , showBaseFM bm
                                 ]

showBaseFM :: BaseFM -> String
showBaseFM = unlines . map f . fmToList
              where f (s,yn) = "   - " ++ s ++ ": " ++ show yn

overview :: String -> BaseFM -> String
overview s bm = "[" ++ (show $ ok bm) ++ " richtig" 
	      ++ ", " ++ (show $ no bm) ++ " falsch"
	      ++ ", " ++ (show $ ok bm + no bm) ++ " Einsendung(en)"
              ++ ", " ++ (show $ length $ keysFM bm) ++ " " ++ s
	      ++ "]"


---------------------------------------------------------------------------
-- Tabellen ausgeben:
---------------------------------------------------------------------------

         -- Breite eines Eintrags 
table :: Int -> InfoFM -> InfoFM -> String
table n am ma = unlines $ 
      [ concat $ map (sep n) ("" : keysFM am) ]
      ++ map (tableLine n ( (n-1) `div` 2 ) ma $ keysFM am) (keysFM ma)

tableLine :: Int -> Int -> InfoFM -> [String] -> String -> String
tableLine n m ma aufs user = 
    sep n user 
    ++ concat (map (sep n . tableItem n m ma user) aufs)

tableItem :: Int -> Int -> InfoFM -> String -> String -> String
tableItem n m ma user auf = 
    let yn = lookupWithDefaultFM 
	     (lookupWithDefaultFM ma emptyFM user) (0,0) auf
    in case yn of
       (0,0) -> stretch (m+m+1) ""
       _     -> (stretch m $ show $ fst yn) ++ " " 
		++ (stretch m $ show $ snd yn)

---------------------------------------------------------------------------
-- Ab gehts:
---------------------------------------------------------------------------

main :: IO ()
main = do
    args <- getArgs

    einschreib <- readFile $ args !! 0

    contents <- mapM readFile $ drop 1 args

    let (ma,am) = mkFMs $ filter ( (>1024) . read . matrikel ) 
			$ concat 
		        $ map ( map readInfo )
			$ map lines contents

    let o = sum $ map ok $ eltsFM ma
    let n = sum $ map no $ eltsFM ma

    zeit <- zeit

    putStr $ unlines 
	 [ heading $ "autotool -- Statistik, Stand von: " ++ zeit
	 , "Eingeschriebene User: " ++ (show $ length $ lines einschreib)
         , "Einsendungen insgesamt: " ++ (show $ o + n)
         , "Einsendungen mit richtiger Lsung: " ++ (show o)
         , "Einsendungen mit falscher Lsung: " ++ (show n)
	 , "Verschiedene User: " ++ (show $ length $ keysFM ma)
         , "Verschiedene Aufgaben: " ++ (show $ length $ keysFM am)
         , ""
	 , heading "User x Aufgabe (OK NO)", table 7 am ma
--	 , heading "Aufgabe x User (OK NO)", table 7 ma am
--	 , heading "nach Aufgabe:", showInfoFM "User" am
--	 , heading "nach User:", showInfoFM "Aufgabe(n)" ma
	 ]

