module Component.Pool where

import Component.Type
import Component.Ident
import Expression

import FiniteMap
import Maybe ( fromMaybe )
import Monad ( guard )
import IOExts

type Components = [ Component ]

type Pool = IORef ( FiniteMap Ident Component )

make :: [ Component ] -> IO Pool
make cs = do
     let fm = listToFM $ do { c <- cs ; return ( ident c, c ) }
     newIORef fm

execute :: Pool -> Update -> IO ()
execute p up = do
    fm <- readIORef p
    let neu    = listToFM $ do { c <- new up ; return ( ident c, c ) }
    let anders = listToFM $ do
	    c <- change up
	    let i = ident c
	    let prev = fromMaybe (error "Pool.execute") $ lookupFM fm i
	    return ( i, override prev c )

    putStrLn $ "execute.anders: " ++ show anders

    writeIORef p 
	$ flip plusFM neu
        $ flip plusFM anders
        $ delListFromFM fm ( map ident $ delete up )


-----------------------------------------------------------------------

logout :: Expression
logout = Id "Logout"

answer :: Pool -> Expression -> IO Expression

answer p (Id "Hello") = do
       fm <- readIORef p
       return $ Positional "Components" 
			   [ List $ map toExp $ keysFM fm ]

answer p (Id "Logout") = do
       return logout

answer p (Named "Component" binds) = do
       fm <- readIORef p
       let i = fromMaybe (error "Pool.ident") $ lookup "ident" binds
       return $ case lookupFM fm $ fromExp i of
		Nothing  -> Id "Unknown"
		Just c -> toExp c

answer p click @ (Named "Click" binds) = do
       fm <- readIORef p
       let i = fromMaybe (error "Pool.ident") $ lookup "ident" binds
       case lookupFM fm $ fromExp i of
		Nothing  -> return $ Id "Unknown"
		Just receiver -> do
		     up <- action receiver click
		     execute p up
		     return $ toExp up

answer d sonst = do
       return $ Id "Unknown"


       







