module Component.Type where

import Component.Option
import Component.Ident
import Component.Point


import Expression
import ToDoc hiding ( text )
import Maybe
import Monad ( guard )
import List ( deleteFirstsBy )

--------------------------------------------------------------------------

type RGB = (Int, Int, Int)

data Option 
     = Position Point
     | Text String
     | Color RGB

data Kind = Label 
	  | Button
     deriving (Eq, Ord, Show, Read)

data Component =
     Component { kind :: Kind
	       , ident :: Ident
	       , options :: [ Option ]
	       , action :: Expression -> IO Update
	       }

data Update = Update { new    :: [ Component ]
		     , change :: [ Component ]
		     , delete :: [ Component ]
		     }

-----------------------------------------------------------------------


instance Expressive Component where
    toExp c = 
	  Named ( show $ kind c ) $
	      -- action wird nicht ausgegeben
	      ( "ident", toExp $ ident c ) : do
	          o <- options c
		  return $ case o of
		       Position p -> ( "position" , toExp p )
		       Text s     -> ( "text"     , toExp s )
		       Color rgb  -> ( "color"    , toExp rgb )

    fromExp (Named k binds) = 
          Component 
	      { kind = read k
	      , ident = fromExp
			$ fromMaybe (error "Label.ident") 
			$ lookup "ident" binds
	      , options = do
    		  (n, v) <- binds
		  guard $ n /= "ident" -- hammer nmlich schon
		  return $ case n of
		       "position" -> Position $ fromExp v
		       "text"	  -> Text     $ fromExp v
		       "color"	  -> Color    $ fromExp v
	      -- action kann man nicht einlesen
	      }

instance ToDoc Component where    
    toDoc = toDoc . toExp

instance Show Component where
    show = showExp

instance Read Component where
    readsPrec = readExp

---------------------------------------------------------------------------

override :: Component -> Component -> Component
-- die optionen (und action!) der zweiten berschreiben die ersten
override x y = 
    let Named xname alt = toExp x
	Named yname neu = toExp y
	bleibt = deleteFirstsBy ( \ (a,_) (b,_) -> a == b ) alt neu
	helper = fromExp $ Named yname $ neu ++ bleibt
    in	y { options = options helper }

with_options :: Component -> [ Option ] -> Component
with_options c os = override c ( c { options = os } )


---------------------------------------------------------------------------

instance Expressive Update where
    toExp u = Named "Update"
	  [ ( "new"    , List $ map toExp $ map ident $ new    u )
	  , ( "change" , List $ map toExp $ map ident $ change u )
	  , ( "delete" , List $ map toExp $ map ident $ delete u )
	  ]
    fromExp ( Named "Update" binds ) = error "Component.Type.Update.read"

instance ToDoc Update where    
    toDoc = toDoc . toExp

instance Show Update where
    show = showExp

-- instance Read Update where
--     readsPrec = readExp

no_update :: Update
no_update = Update { new = [], change = [], delete = [] }

--------------------------------------------------------------------------




