// FILE: Protocol.java

import java.net.*;
import java.io.*;


 
public class Protocol
{
	private Socket server;
	private InputStream in;
	private OutputStream out;
	private boolean connected;
	
	
	public Protocol()
	{
		connected = false;
	}
	
	public boolean openConnection()
	{
		if (connected) return true;
		
		try 
		{
			server = new Socket(ServerData.host, ServerData.portNumber);

			in  = server.getInputStream();
			out = server.getOutputStream();
		}
		catch (UnknownHostException e)
		{
			System.out.println("Host "+ServerData.host+" nicht gefunden.");
			System.out.println(e);
			return false;
		}
		catch (IOException e)
		{
			System.out.println("Input-/Output-Streams konnten nicht erzeugt werden.");
			System.out.println(e);
			
			return false;
		}
		
		connected = true;
		System.out.println("Verbindung aufgebaut.");
		return true;
	}
	
	public boolean closeConnection()
	{
		try
		{
			sendToServer ("Logout");
			server.close();
		}
		catch (IOException e)
		{
			System.out.println("Verbindung zum Host "+ServerData.host+" konnte nicht abgebaut werden.");
			System.out.println(e);
			
			return false;
		}
		
		System.out.println("Verbindung abgebaut.");
		return true;
	}
	
	public String getAnswer()
	{
		String answer = new String();
		byte b[]=new byte[256];
		
		try
		{
			in.read(b);
			//for(int i=0; i<b.length; i++) System.out.print((char)b[i]);
			
			answer = new String(b);
		}
		catch (IOException e)
		{
			System.out.println("Die Antwort vom Server konnte nicht gelesen werden.");
			System.out.println(e);
			return null;
		}
		
		return answer;
	}
	
	public boolean sendToServer(String sendString)
	{
		sendString = sendString + "\n";

		byte sendBytes[];
		
		sendBytes = sendString.getBytes();
		
		try
		{
			out.write(sendBytes);
			//for(int i=0; i<sendBytes.length;i++)
			//	System.out.print((char)sendBytes[i]);
		}
		catch (IOException e)
		{
			System.out.println("String "+sendString+" konnte nicht bermittelt werden.");
			System.out.println(e);
			
			return false;
		}
		
		return true;
	}

	//legt ein Pause von time Millisekunden ein
	public void pause(int time)
	{
		try {
			Thread.sleep(time);
		} catch (InterruptedException e) { }
	}


}
