-- | Verwaltung fuer Beweise (Loesungen) von einen Problem
--  ablegen in Dateien usw.
--
-- autor Georg Martius
-- mai99dgf@studserv.uni-leipzig.de


module Beweise (
    Beweis (..)
  , speichere
  , lade
  , getAlle
  , getAlleNachString
  , addToAlle
  , nurKorrekte 
  , nurKorrekte1ProAutor 
  , module ToDoc
  , module Zeit
  ) where

import Challenger hiding (ident, instanz)
import qualified Challenger

import ToDoc
import Zeit
import Konfiguration
import qualified Datei as D


-- | Datenstruktur fuer einen Beweis
data Beweis = Beweis { 
      instanz :: Ident
    , ident :: Int 
	  , korrekt :: Bool 
    , autor :: String
    , kommentar :: String
    , datum :: CalendarTime    
    } deriving (Show,Read)


-- | liefert alle korrekten Beweise aus der bergebenen Liste zurck
nurKorrekte :: [Beweis] -> [Beweis]
nurKorrekte liste = filter (\ x -> (korrekt x) == True) liste 

-- | liefert pro Autor den ersten korrekten Beweis aus der bergebenen Liste
nurKorrekte1ProAutor :: [Beweis] -> [Beweis]
nurKorrekte1ProAutor liste = onlyFirstBeweise (nurKorrekte liste)

-- | den nur den ersten Beweis jedes Autors in die Liste aufnehmen
-- (1) Liste von Beweisen
-- return Liste von "ersten" Beweisen (keine doppelten Autoren)
onlyFirstBeweise:: [Beweis]->[Beweis]
onlyFirstBeweise [] = []
onlyFirstBeweise (a:l) =
  a: (onlyFirstBeweise (filter (\ x -> autor x /= autor a) l))

-- | generiert eine neue Identifikation des Beweises fuer die Instanz
--  letzter Beweis in beweiseXXX.list + 1
neu :: (Show p) => p -> Ident -> IO (Int)
neu p instanz =  do
    str <- catch (D.lesen (beweisListFileName (show p) instanz)) 
                     ( \ any -> return "" )
    let zeilen = lines str
    return $ (length zeilen)  

-- | speichert eine neuen Beweis zur Instanz auf die Platte
-- fuegt ihn zur Liste der Beweise hinzu und  
-- generiert eine neue Identifikation und gibt sie zurck  
-- Dateiname ist Id vom letzten Beweis in "beweiseXXX.list" + 1 als 3_3-stelliger String
-- Bsp: 001_005.beweis Der 5.Beweis zur 1. Instanz 
-- (1) Einsendung (muss Loesung sein!)
-- (2) Autor
-- (3) ist der Beweis korrekt 
-- return Id des Beweises
speichere :: Problem p i b => Einsendung p i b -> String -> Bool -> IO (Int)
speichere (p @ Loesung {}) aut korr  = do
    id <- neu (problem p) (Challenger.ident p)
    dat <- zeit
    addToAlle (problem p) (Challenger.ident p) 
                Beweis {
                  instanz = (Challenger.ident p)
                , ident = id  
                , korrekt = korr 
                , autor = aut                
                , kommentar = ""
                , datum = dat                
                }
    D.schreiben (beweisFileName (problem p) (Challenger.ident p) id) (render $ toDoc $ beweis p)
    return id

speichere (p @ Aufgabe {}) aut korr = do
    return (-1);

-- | Ladt einen Beweis
-- (1) Problem
-- (2) Ident des Instanz
-- (3) Id des Beweises
-- return Beweis
lade :: (Problem p i b) => Einsendung p i b -> Ident -> Int -> IO b
lade e instanz id= do
    string <- D.lesen (beweisFileName (problem e) instanz id ) 
    let bew = read string
    return bew 

-- | Der Dateiname mit Pfad einer Beweisdatei 
-- (1) Problem
-- (2) Ident des Instanz
-- (3) Id des Beweises
-- return Dateiname mit Pfad
beweisFileName :: (Show p)=> p -> Ident -> Int -> D.Datei
beweisFileName p instanz id = 
  D.Datei { D.pfad = [ challengerWurzelverz
                     , "aufgaben"
                     , show p
                     , (stringMitNullen (showIdentForDatei instanz) 4)
                     , "beweis" ++ (stringMitNullen (show id) 4)
                     ]
          , D.name = "beweis"                     
          } 

-- | Laedt alle Beweise zur angegebenen Instanz des Problems
-- (1) Problem
-- (2) Ident der Instanz
-- return Liste der Beweise
getAlle :: (Show p) => p -> Ident -> IO ([Beweis])
getAlle p instanz= do 
    str <- catch (D.lesen (beweisListFileName (show p) instanz)) 
                     ( \ any -> return [] )
    let zeilen = lines str
    let beweise = map read zeilen
    return beweise

-- | Laedt alle Beweise zur angegebenen Instanz des Problems
-- (1) Problem ( als String )
-- (2) Ident der Instanz
-- return Liste der Beweise
getAlleNachString :: String -> Ident -> IO ([Beweis])
getAlleNachString p instanz = do 
    str <- catch (D.lesen (beweisListFileName p instanz)) 
                     ( \ any -> return [] )
    let zeilen = lines str
    let beweise = map read zeilen
    return beweise
         
-- | Speichert alle Beweise zur angegebenen Instanz des Problems
-- (1) Problem
-- (2) Ident der Instanz
-- (3) Liste der Beweise, die zu schreiben sind
-- return 
setAlle :: (Show p) => p -> Ident -> [Beweis] -> IO()
setAlle p instanz lsglist = do
    let zeilen = map show lsglist
    let str = unlines zeilen
    D.schreiben (beweisListFileName (show p) instanz) str

-- | Fuegt einen Beweis in die Liste der Beweise hinzu
-- (1) Problem
-- (2) Ident der Instanz
-- (3) Beweis 
addToAlle :: (Show p) => p -> Ident -> Beweis -> IO()
addToAlle p instanz bew  = do
   catch (D.anhaengen (beweisListFileName (show p) instanz) (show (bew) ++ "\n")) 
                  ( \ any -> D.schreiben (beweisListFileName (show p) instanz) (show (bew) ++ "\n")) 

-- | Dateiname der Listendatei fuer Beweise
-- (1) Problem (als String)
-- (2) Instanz
-- return Datei
beweisListFileName :: String -> Ident -> D.Datei
beweisListFileName p instanz = 
    D.Datei { D.pfad =  [ challengerWurzelverz
                        , "aufgaben"
                        , p
                        , (stringMitNullen (showIdentForDatei instanz) 4)
                        ]
            , D.name = "beweise.list"
            }


-- | String wird auf n Zeichen aufgeblaeht (wenn laenger als n dann nichts)
-- (1) String, der aufgeblaeht werden soll
-- (2) Anzahl der Ziffern
-- return neuer String
stringMitNullen :: String -> Int -> String
stringMitNullen s len = 
    if length s < len 
        then
          (replicate (len - length s) '0') ++ s
        else   
          s


