-- | Die Datenstrukturen fuer Challengeraufgaben
--   (zu importieren von Student) 
--
-- autor Georg Martius
-- mai99dgf@studserv.uni-leipzig.de


module Challenger (
    module ToDoc
  , module Iso
  , module Number
  , Problem (..)
  , Einsendung (..)
  , ident
  , instanz
  , beweis
  , Ident (..)
  , showIdentForDatei
  )
  where

import ToDoc
import Iso
import Number
import System

import Graph.Type

-- | Klasse Problem steht fr ein abstraktes Problem zu dem
-- Instanzen mit Beweisen existieren und auf Korrektheit 
-- ueberprueft werden koennen
--  validiere ist zum vorpruefen der Instanz gedacht
--  verifiziere dann zum ueberpruefen der Loesung (des Beweises)
class (Show p, ToDoc i, Show i, Read i
  , ToDoc b, Show b, Read b)
    => Problem  p i b | p b -> i  where
       validiere   :: p -> i -> b -> (Bool, Doc) 
       verifiziere :: p -> i -> b -> (Bool, Doc) 

       -- String ist der Dateiname ohne Endung 
       -- return: datei, Typ (endung), ExitCode des Systemaufrufs
       getInstanz :: p -> i -> b -> String -> IO(String, String, ExitCode)
       getBeweis  :: p -> i -> b -> String -> IO(String, String, ExitCode)

------------------------------------------------------------------------------

data Einsendung p i b
    = Aufgabe { problem::p, instanz::i, beweis::b}
    | Loesung { problem::p, ident :: Ident , beweis::b}
    
instance (Show p, ToDoc i, ToDoc b) => ToDoc (Einsendung p i b) where 
  toDoc (e @ Aufgabe {}) = text "Aufgabe" <+> braces ( 
      fsep $ punctuate comma
      [ text "problem" <+> equals <+> text ( show (problem e))
      , text "instanz" <+> equals <+> toDoc (instanz e)
      , text "beweis" <+> equals <+> toDoc (beweis e)
      ])
  
  toDoc (e @ Loesung {}) = text "Loesung" <+> braces ( 
       fsep $ punctuate comma
       [ text "problem" <+> equals <+> text ( show (problem e))
       , text "ident" <+> equals <+> text ( show (ident e))
       , text "beweis" <+> equals <+> toDoc (beweis e)
       ]
      )
   

instance ToDoc (Einsendung p i b) => Show (Einsendung p i b) where 
  show = render . toDoc
    
data Ident = Ident {aufgabe :: Int } deriving (Show, Read)

showIdentForDatei :: Ident -> String
showIdentForDatei i = show (aufgabe i)
  
