-- | Korrekturfunktion fr Cliquen-Aufgaben
--
-- autor Markus Kreuz
-- mai99byv@studserv.uni-leipzig.de

-- patches: joe@informatik.uni-leipzig.de

module Clique.Clique 
( module Graph.Type
, Clique
, module Set
) where

import Graph.Type
import Graph.Util
import Graph.Viz
import Graph.Valid
import Challenger
import Number

import ToDoc
import Set
import List (tails)
import Monad (guard)

-- $Log: Clique.hs,v $
-- Revision 1.7  2002/06/29 12:00:44  joe
-- instance Number (Graph a, Integer)
--
-- Revision 1.6  2002/06/29 11:49:16  joe
-- cliquen sind jetzt mengen
--

data Clique = Clique deriving Show

-- Instanzierung der Klasse Problem (problem, instanz, beweis)
--   eine Instanz muss hier ein (Graph a) sein, also ein Graph mit beliebigem
--     Knotentyp
--   ein Beweis ist eine Menge von Knoten (eine Clique)

instance Ord a => Number (Graph a, Integer) (Graph Int, Integer) where
    number (g, s) = (number g, s)

instance (ToDoc ((Graph a), Integer), Show ((Graph a), Integer), Read ((Graph a), Integer)
         , ToDoc (Set a), Show (Set a), Read (Set a)
	 , Ord a, ToDoc a , Show a
	 , ToDoc [a]
	 )
    => Problem Clique ((Graph a), Integer) (Set a) where

    validiere Clique (g, anzahl) lsg = valid g

    verifiziere Clique (g, anzahl) lsg = 
        let aussen = minusSet lsg (knoten g)
	    nokanten = do
	        (x : ys) <- tails $ setToList lsg
		y <- ys
		let k = kante x y
		guard $ not $ elementOf k $ kanten g
		return k
	in  if not $ isEmptySet aussen
	    then ( False
		 , text "Diese Knoten gehren berhaupt nicht zum Graphen:"
			  <+> toDoc aussen 
		 )
	    else if fromIntegral (cardinality lsg) < anzahl
	    then ( False
		 , fsep [ text "Die Knotenmenge enthlt nur"
			, toDoc (cardinality lsg), text "Knoten,"
			, text "es sollen aber wenigstens"
			, toDoc anzahl, text "sein."
			]
		 )
	    else if not $ null nokanten
	    then ( False
		 , fsep [ text "Die Knoten-Teilmenge ist keine Clique,"
			, text "denn diese Kanten fehlen im Graphen:"
			, toDoc nokanten
			]
		 )
	    else ( True
		 , text "Das ist eine Clique der geforderten Gre."
		 )
    
    getInstanz Clique (graph, anzahl) clique dateiName =
	-- mehr muss hier keiner machen, der Graphviz nutzt
    	getGraphviz graph instanzTrans dateiName

    getBeweis Clique (graph, anzahl) clique dateiName =
    	getGraphviz graph (beweisTrans clique) dateiName
        

-------------------------------------------------------------------------------
-- hier folgen Transformationen
-------------------------------------------------------------------------------
-- ganz einfache Transformation

instanzTrans :: Show knoten => GVTrans knoten
instanzTrans = GVTrans
	{ getGVProg = Default
	, getGVFormat = "png"
	, isGVDirected = False
	, getGVNID = show		-- Graphviz Knoten ID ist einfach show knoten
	, getGVNName = show		-- Knotenname auch
	, getGVNLabel = Nothing
	, getGVNColor = Nothing
	, getGVNXAtts = Nothing
	, getGVELabel = Nothing
	, getGVEXAtts = Nothing
	}


beweisTrans :: (Show knoten, Ord knoten) 
	=> (Set knoten) -> (GVTrans knoten)
beweisTrans knoten = GVTrans
	{ getGVProg = Default
	, getGVFormat = "png"
	, isGVDirected = False
	, getGVNID = show		-- Graphviz Knoten ID ist einfach show knoten
	, getGVNName = show		-- Knotenname auch
	, getGVNLabel = Nothing
	, getGVNColor = Just(getfarbe knoten) -- Knoten der Clique werden Rot gezeichnet
	, getGVNXAtts = Nothing
	, getGVELabel = Nothing
	, getGVEXAtts = Nothing
	}


-- Waehlt die Knoten aus, die eingefaerbt werden
getfarbe :: (Ord knoten) => Set knoten -> knoten -> GVColor
getfarbe clique k = if elementOf k clique then "Red" else "Black" 




--Beispielstrukturen
bsp_graph = Graph {
    knoten = mkSet [0, 1, 2, 3, 4, 5, 6],
    kanten = mkSet [kante 0 1, kante 0 2, kante 1 0, kante 1 2, kante 2 0, kante 2 1]
}
bsp_clique = mkSet [0,1,2]

