-- | Korrekturfunktion fr Hamiltonkreis-Aufgaben
--
-- autor Markus Kreuz
-- mai99byv@studserv.uni-leipzig.de

-- patches: joe@informatik.uni-leipzig.de

module Hamilton.Hamilton (
	  Hamilton
--        , Kreis (..)
	, module Graph.Type
	, validiere
	, verifiziere) 
  	where

import Graph.Type
import Graph.Util
import Graph.Viz
import Graph.Valid
import Graph.Labeling (symdiff)

import Challenger
import ToDoc
import Set
import List (tails)
import Monad (guard)


data Hamilton = Hamilton deriving Show

-- data Kreis a = Kreis [a] deriving (Show, Read)
-- instance (ToDoc [a])=> ToDoc (Kreis a) where
--	toDoc (Kreis krs) = text "Kreis: " <+> toDoc krs

type Kreis a = [ a ]




-- Instanzierung der Klasse Problem (problem, instanz, beweis)
--   eine Instanz muss hier ein (Graph a) sein, also ein Graph mit beliebigem
--     Knotentyp
--   ein Beweis muss hier ein (Kreis a) sein, also eine Liste vom Knotetyp a
 



instance (ToDoc (Graph a), Show (Graph a), Read (Graph a)
         , ToDoc (Kreis a), Show (Kreis a), Read (Kreis a), Ord a, ToDoc a
         , Show a)
    => Problem Hamilton (Graph a) (Kreis a) where

    validiere Hamilton g lsg = 
        Graph.Valid.valid g


    verifiziere Hamilton g lsg =
        let ft @ (f, t) = einfach (knoten g) lsg
	    kreis = mkSet $ do
	        (x, y) <- zip lsg (last lsg : lsg)
		return $ kante x y
	    fehlt = minusSet kreis (kanten g)
	in  if not f then ft
	    else if not $ isEmptySet fehlt
	    then ( False, text "diese Kanten sind nicht im Graphen:"
			  <+> toDoc fehlt )
	    else ( True, text "Das ist ein Hamilton-Kreis im Graphen.")


    getInstanz Hamilton graph labeling dateiName =
		getGraphviz graph getInstanzTrans dateiName


    getBeweis Hamilton graph kreis dateiName =
		getGraphviz graph (getBeweisTrans kreis) dateiName


einfach :: ( Ord a, ToDoc [a] )
	=> Set a -> [a] -> ( Bool, Doc )
einfach s xs = 
    let ( fehlt, zuviel ) = symdiff s (mkSet xs)
	mehrfach = do x : ys <- (tails xs)
		      guard $ elem x ys
		      return x
    in	if not $ isEmptySet fehlt
	then ( False, text "Diese Elemente kommen nicht in der Liste vor:"
		      <+> toDoc fehlt )
	else if not $ isEmptySet zuviel
	then ( False, text "Diese Listen-Elemente kommen nicht in der Menge vor:" )
	else if not $ null mehrfach
	then ( False, text "Diese Elemente kommen mehrfach in der Liste vor" )
	else ( True,  text "Die Liste ist eine wiederholungsfreie Anordnung der Menge.")



-------------------------------------------------------------------------------
-- hier folgen Transformationen
-------------------------------------------------------------------------------

-- ganz einfache Transformation
getInstanzTrans :: Show knoten => GVTrans knoten
getInstanzTrans = GVTrans
	{ getGVProg = Default
	, getGVFormat = "png"
	, isGVDirected = False
	, getGVNID = show		-- Graphviz Knoten ID ist einfach show knoten
	, getGVNName = show		-- Knotenname auch
	, getGVNLabel = Nothing
	, getGVNColor = Nothing
	, getGVNXAtts = Nothing
	, getGVELabel = Nothing
	, getGVEXAtts = Nothing
	}

-- ganz einfache Transformation
getBeweisTrans :: (Show knoten, Eq knoten) => Kreis knoten -> GVTrans knoten 
getBeweisTrans kreis = GVTrans 
	{ getGVProg = Default
	, getGVFormat = "png"
	, isGVDirected = False
	, getGVNID = show		-- Graphviz Knoten ID ist einfach show knoten
	, getGVNName = show		-- Knotenname auch
	, getGVNLabel = Nothing
	, getGVNColor = Nothing
	, getGVNXAtts = Nothing
	, getGVELabel = Nothing
	, getGVEXAtts = Just (kantenAttribs kreis)
	}


kantenAttribs ::  (Eq a) => Kreis a -> Kante a -> GVXAtts
kantenAttribs (kreis) kante = 
    if istDrin (kreis ++ [(head kreis)]) (von kante) (nach kante) 
        then [("style","bold"), ("color","red")]
        else [("style","filled")]
    where
        istDrin :: (Eq a) => [a] -> a -> a -> Bool
        istDrin (kn:[]) v n = False
        istDrin (kn:kreis) v n = 
            if (v == kn) && (n == (head kreis)) 
                then True
                else istDrin kreis v n
        istDrin [] v n = False

    



--Beispielstrukturen
bsp_graph = Graph {
    knoten = mkSet [0, 1, 2, 3],
    kanten = mkSet [kante 0 1, kante 1 2, kante 2 3, kante 3 0, kante 1 3]
}

bsp_kreis = [0,1,2,3]


