-- | Verwaltung fuer Instanzen von einen Problem
--  ablegen in Dateien usw.
--
-- autor Georg Martius
-- mai99dgf@studserv.uni-leipzig.de


module Instanzen (
    Instanz (..)
  , speichere
  , speichereInt  
  , lade
  , ladeAlleInt
  , getAlle
  , getAlleNachString
  , addToAlle
  , nurKorrekte
  , getAutor
  , module Zeit
  , module ToDoc
  ) where

import Challenger  hiding (ident)
import qualified Challenger
import ToDoc
import Zeit
import Problems
import Konfiguration
import qualified Datei as D 

-- | Datenstruktur fuer eine Instanz
data Instanz = Instanz { 
      ident :: Ident
    , korrekt :: Bool
    , autor :: String
    , kommentar :: String
    , flag	:: Int  -- unused now
    , datum :: CalendarTime 
    } deriving (Show, Read)


nurKorrekte :: [Instanz] -> [Instanz]
nurKorrekte liste = filter (\ x -> (korrekt x) == True) liste 

    
-- | generiert einen neuen Ident zum identifizieren der Aufgabe
--  letzte Instanz in instanzen.list + 1
neu :: (Show p) => p -> IO (Ident)

neu p =  do
    str <- catch (D.lesen (instanzListFileName $ show p)) 
                 ( \ any -> return "")
    let zeilen = lines str
    return (Ident {aufgabe = (length zeilen + 1)})  

getAutor :: (Show p) => p -> Ident -> IO(String)
getAutor prob ident = do
	list <- getAlle prob
	let index = ((aufgabe ident)-1)
	return (if and [(length list) > index , index >= 0 ] then (autor (list
		!! index)) else "Unknown")

-- | speichert eine neue Aufgabe auf die Platte 
-- fuegt sie in die Liste der Instanzen hinzu und
-- generiert neuen Ident und gibt ihn zurck  
--  Dateiname ist Ident von letzter Instanz in "instanzen.list" + 1 als 4-stelliger String
-- (1) Einsendung (muss Aufgabe sein!)
-- (2) Autor
-- (3) ist die Instanz korrekt 
-- return Ident der Instanz
speichere :: Problem p i b => Einsendung p i b -> String -> Bool -> IO (Ident)
speichere (p @ Loesung {}) aut korr = do
    return Ident { aufgabe = -1}

speichere (p @ Aufgabe {}) aut korr = do
    id <- neu (problem p)
    dat <- zeit
    addToAlle (problem p)
       Instanz {
                  ident = id  
                , korrekt = korr 
                , autor = aut
                , kommentar = ""
                , flag  = 0
                , datum = dat
                }
    D.schreiben (instanzFileName (problem p) id) (render $ toDoc $ instanz p)
    return id

-- | speichert eine neue Aufgabe, die von PolyToInt (durchnummeriert) sein muss auf die Platte
-- (1) Problem
-- (2) Ident
-- return 
speichereInt :: (Show p, Number i iint, Show iint) => p -> Ident -> i -> IO ()
speichereInt p id inst = do
    D.schreiben (instanzIntFileName p id) (show $ (number inst))
-- | Ladt eine Instanz
-- (1) Einsendung
-- (2) Ident des Instanz (es koennt sich bei der Einsendung um eine Aufgabe handeln)
-- return Instanz    
lade :: (Problem p i b) => Einsendung p i b -> Ident -> IO i
lade e id = do
    string <- D.lesen (instanzFileName (problem e) id)
    let inst = read string
    return inst 

ladeAlleInt :: (Show p,  Read iint) => p -> iint -> [Ident] -> IO [iint]
ladeAlleInt p norminst [] = do return []
ladeAlleInt p norminst (id:idents) = do
    string <- catch ( D.lesen (instanzIntFileName p id) )
                    ( \ any -> do return "") 
    if(string == []) 
      then 
        ladeAlleInt p norminst idents
      else do
        let inst = read string
        rest <- ladeAlleInt p norminst idents
        return (inst : rest)

  
-- | Der Dateiname mit Pfad einer Instanzdatei 
-- (1) Problem
-- (2) Ident der Instanz
-- return Datei
instanzFileName :: (Show p)=> p -> Ident -> D.Datei
instanzFileName p id = D.Datei{ D.pfad = [ challengerWurzelverz 
                                         , "aufgaben"
                                         , show p 
                                         , (stringMitNullen (showIdentForDatei id) 4)
                                         ]
                              , D.name = "instanz"
                              }

-- | Der Dateiname mit Pfad einer Durchnummerierten Instanzdatei 
-- (1) Problem
-- (2) Ident der Instanz
-- return Datei
instanzIntFileName :: (Show p)=> p -> Ident -> D.Datei
instanzIntFileName p id = D.Datei{ D.pfad = [ challengerWurzelverz 
                                         , "aufgaben"
                                         , show p 
                                         , (stringMitNullen (showIdentForDatei id) 4)
                                         ]    
                                   , D.name = "instanz.number"
                                   }

-- | Laedt alle Instanzen des Problems
-- (1) Problem
-- return Liste der Instanzen
getAlle :: (Show p) => p -> IO ([Instanz])
getAlle p = do
    str <- catch ( D.lesen (instanzListFileName $ show p))
                 ( \ any -> return [] )
    let zeilen = lines str
    let instanzen = map read zeilen
    return instanzen
    
-- | Laedt alle Instanzen des Problems
-- (1) Problem ( als String )
-- return Liste der Instanzen
getAlleNachString :: String -> IO ([Instanz])
getAlleNachString p = do
    str <- catch ( D.lesen (instanzListFileName p))
                 ( \ any -> return [] )
    let zeilen = lines str
    let instanzen = map read zeilen
    return instanzen
 
-- | Speichert alle Instanzen zum Problems
-- (1) Problem
-- (2) Liste der Instanzen, die zu schreiben sind
-- return 
setAlle :: (Show p) => p -> [Instanz] -> IO()
setAlle p instlist = do
    let zeilen = map show instlist
    let str = unlines zeilen
    D.schreiben (instanzListFileName $ show p) str

-- | Fuegt eine Instanz zu der Liste der Instanzen hinzu
-- (1) Problem
-- (2) Instanz
-- return 
addToAlle :: (Show p) => p -> Instanz -> IO()
addToAlle p inst = do
    catch (D.anhaengen (instanzListFileName $ show p) (show (inst) ++ "\n")) 
                  ( \ any -> D.schreiben (instanzListFileName $ show p) (show (inst) ++ "\n")) 

-- | Dateiname der Listendatei fuer Instanzen
-- (1) Problem (als String)
-- return Datei
instanzListFileName :: String -> D.Datei
instanzListFileName p = D.Datei{ D.pfad = [ challengerWurzelverz 
                                         , "aufgaben"
                                         , p 
                                         ]
                               , D.name = "instanzen.list"
                               }

-- | String wird auf n Zeichen aufgeblaeht (wenn laenger als n dann nichts)
-- (1) String, der aufgeblaeht werden soll
-- (2) Anzahl der Ziffern
-- return neuer String
stringMitNullen :: String -> Int -> String
stringMitNullen s len = 
    if length s < len 
        then
          (replicate (len - length s) '0') ++ s
        else   
          s


