-- | Das Kontrollprogramm fuer Challenger Aufgaben 
--    wird direkt vom Manager aufgerufen
--
-- autor Georg Martius
-- mai99dgf@studserv.uni-leipzig.de
-- Beispiele auskommentiert von Alexander Kiel am 28.05.2002 2:29Uhr

module Kontrolleur (
	bewerte
	) where

import Challenger
import qualified Instanzen
import qualified Beweise
import ToDoc
import Zeit
import Problems
import Konfiguration
import IsomorphieTest

import System
import FiniteMap
import ReadFM

import qualified Web.Index as WIndex
import qualified Web.Instanz as WInstanz
import qualified Web.Statistik as WStat

-- nur zum testen? hoffentlich. das hauptprogramm hier ist doch generisch.
import Graph.Type

-- | diese Funktion wird vom Autotool Manager aufgerufen
-- (1) die Einsendung des Studenten


bewerte :: (Problem p i b          
          , Number i iint, Read iint, Show iint, Iso iint) 
          => String -> Einsendung p i b -> IO String
bewerte autor e = do

    putStrLnLn $ render $ fsep 
       [ text "Guten Tag, ich bin der Challenger-Kontrolleur."
       , text "Ihre Einsendung war:"
       , toDoc e
       ]

    (flag,doc) <-  handle e autor

    -- TODO: geht das besser? wo kann man die umask ndern?
    -- procmail ist offenbar sehr restriktiv.
    system "chmod -R go+r $(find /home/challenger/public_html/challenger -type f)"
    system "chmod -R go+xr $(find /home/challenger/public_html/challenger -type d)"    

    putStrLnLn $ render $ doc

    if flag then return "OK"
	    else return "NO"
		

-- | speichert die komplette Einsendung ab
-- (1) Eindenung 
-- (2) Autor
-- (3) korrekt
-- return Instanz nummer, Beweis nummer 
speichereEinsendung :: (Problem p i b, Number i iint, Show iint) => Einsendung p i b -> String -> Bool -> IO (Ident,Int)
-- | fuer Aufgaben: speichert alles
speichereEinsendung (e @ Aufgabe {}) autor korrekt = do
   instnr <- Instanzen.speichere e autor korrekt
   Instanzen.speichereInt (problem e) instnr (instanz e)
   bewid <- Beweise.speichere 
  	 		Loesung {
      				  problem = problem e
    					, ident   = instnr
    					, beweis  = beweis e 
  						} 
   			autor korrekt
   return (instnr, bewid)          

                      
-- | fuer Loesung: speichert nur Beweis
speichereEinsendung (e @ Loesung {}) autor korrekt =  do
	bewid <- Beweise.speichere e autor korrekt
	return (ident e, bewid)
     		  
-- | Behandelt die Einsendung
-- Speichert alles ab und gibt das Dokument fuer den Einsender zurueck
-- (1) Einsendung
-- (2) Autor
-- return Dokument
handle :: (Problem p i b
          , Number i iint, Read iint, Show iint, Iso iint) 
          => Einsendung p i b -> String -> IO (Bool, Doc)
handle (e @ Aufgabe {}) autor = do
-- test auf isomorphie mit alle bisherigen Loesung
    (flag, instid) <- isomorphieTest e
    if flag 
      then
		do 
        return ( False, fsep
	       [ text $ "Es existiert bereits eine dazu isomorphe Aufgabe"
	       , text $ "unter der Nummer" ++ showIdentForDatei instid ++ "."
               ] )
      else do  
        -- Instanz validieren lassen
        (flag, validdoc) <- makeIO $ validiere (problem e) (instanz e) (beweis e)
        if not flag 
          then do
            (instid,bewid)<- speichereEinsendung e autor False    
            return (False, fsep  
			[ text "Die Aufgabe wurde nicht angenommen,"
			, text "weil die Instanz nicht gueltig ist."
			, text "Hier die Ausgabe des Korrekturprogramms:"
			, validdoc 
			] )      
          else 
            do
            -- Loesung verifizieren lassen   
            (flag, veridoc) <- makeIO $ verifiziere (problem e) (instanz e) (beweis e)    
            if not flag 
              then do
                (instid,bewid)<- speichereEinsendung e autor False
                return (False, fsep  
			[ text "Die Aufgabe wurde nicht angenommen,"
			, text "weil die Lsung nicht korrekt ist."
			, text "Hier die Ausgabe des Korrekturprogramms:"
			, veridoc 
			] )   
              else do
                -- Ok die Aufgabe ist in Ordnung 
                -- -> abspeichern
                -- -> Html-zeug durchlaufen lassen 
                (instid,bewid)<- speichereEinsendung e autor True
                WStat.problemStatistik (show (problem e))
                WInstanz.erzeugeInstanzDatei e instid autor False 
                WIndex.erzeugeIndex
                return (True, fsep  
			[ text "Herzlichen Glckwunsch."
			, text "Die Aufgabe wurde akzeptiert."
			, text "Hier die Ausgaben des Korrekturprogramms:"
			, validdoc
			, veridoc
			] )   

handle (e @ Loesung {}) autor = do

     -- Loesung verifizieren lassen 
     inst <- Instanzen.lade e (ident e)

     putStrLnLn $ render $ fsep [ text "Die Instanz war", toDoc inst ]

     (flag, veridoc) <- makeIO $ verifiziere (problem e) inst (beweis e)    
     if not flag 
       then do 
        (instid,bewid)<- speichereEinsendung e autor False          
        return (False, fsep 
		      [ text "Die Loesung ist leider nicht korrekt."
                      , text "Hier die Ausgabe der Kontrollroutine:"
                      , veridoc
		      ] )

       else do
         -- neue Loesung speichern
        (instid,bewid)<- speichereEinsendung e autor True
        WStat.problemStatistik (show (problem e))
        -- 
        beweise <- Beweise.getAlle (problem e) (ident e)
        let anzahlKorr = length $ Beweise.nurKorrekte1ProAutor beweise
        if (anzahlKorr > anzahlRichtigerBeweiseBisLoesung) 
          then do
            origautor  <- Instanzen.getAutor (problem e) (ident e)
            origbeweis <- Beweise.lade e (ident e) 0  
          
            WInstanz.erzeugeInstanzDatei
                 Aufgabe { problem = problem e,
                                      instanz = inst,
                                      beweis = origbeweis}
  		          (ident e) origautor True	                 

            return ""
          else do return ""    
         
        return (True, fsep 
		      [ text "Herzlichen Glckwunsch." 
		      , text "Die Loesung ist korrekt."
                      , text "Hier die Ausgabe der Kontrollroutine:"
                      , veridoc
		      ] )
	       
----------------------------------------------------------------------------

-- Hilfsfunktionen
makeIO :: a -> IO(a)
-- das IST return (siehe class Functor, instance Functor IO)
makeIO x = do
    return x

makeUnIO :: (a -> b) -> IO(a) -> IO(b)
-- das IST fmap (siehe class Functor, instance Functor IO)
makeUnIO f a = do
    val <- a
    return $ f val 

putStrLnLn :: String -> IO ()
putStrLnLn cs = do putStrLn cs ; putStrLn ""

------------------------------------------------------------------------------

-- test zeug

m = do
   doc <- handle test_Ceinsendung "8888885"
   putStr $ show doc

l = do
   doc <- handle test_CLeinsendung "123"
   putStr $ show doc

 
test_Geinsendung = Aufgabe { problem = Graceful, instanz = bsp_graph , beweis = bsp_labeling }
test_GLeinsendung = Loesung{ problem = Graceful, ident = Ident {aufgabe = 2 } , beweis = bsp_labeling }

bsp_graph = Graph{ knoten = mkSet [1,2,3]
				 , kanten = mkSet [kante 1 2
                				  ,kante 1 3
                                  ]
                 }
bsp_graph2 = Graph{ knoten = mkSet ['a','b','c']
				  , kanten = mkSet [kante 'b' 'c'
                				  ,kante 'b' 'a'
                                  ]
                 }
                
bsp_labeling = listToFM [(1,0),(2,1),(3,2)]
                


test_Ceinsendung = Aufgabe { problem = DreiCol, instanz = bsp_Cgraph , beweis = bsp_Cfaerb }
test_CLeinsendung = Loesung{ problem = DreiCol, ident = Ident {aufgabe = 1 } , beweis = bsp_Cfaerb }
bsp_Cgraph = Graph {
    knoten = mkSet [1, 2, 3, 4, 5],
    kanten = mkSet [kante 1 2, kante 2 3, kante 3 4, kante 4 5, kante 3 5, kante 1 5]
}

-- bsp_Cfaerb = Klassen [[1,4],[2,5],[3]]
bsp_Cfaerb = listToFM [(1,1),(2,2),(3,3),(4,1),(5,2)]




-- test_Heinsendung = Aufgabe { problem = Hamilton, instanz = bsp_Hgraph , beweis = bsp_Hkreis }
-- test_HLeinsendung = Loesung{ problem = Hamilton, ident = Ident {aufgabe = 1 } , beweis = bsp_labeling }



-- Aufgabe
{- m0a = do 
    doc <- handle test_GLeinsendung "Georg"
    putStr $ show doc
    
    

m3 = do
      list <- Instanzen.getAlle Graceful
      putStr $ show list       
      
m3a = do
      dat <- zeit
      rval <- Instanzen.addToAlle Graceful  
          Instanzen.Instanz { Instanzen.datum = dat
                            , Instanzen.ident = Ident {aufgabe = 2}
                            , Instanzen.autor = "mustermann"
                            , Instanzen.korrekt = True
                            }
      list <- Instanzen.getAlle Graceful
      putStr $ show list       
        
  
m5 = do
      list <- Beweise.getAlle Graceful Ident{aufgabe = 1}
      putStr $ show list       
 
m6 = do
      rval <- Beweise.speichere test_CLeinsendung "Georg" True
      list <- Beweise.getAlle Col Ident{aufgabe = 1}              
      putStr $ show list       
-}
