-- | erzeugt die komplette Html Representation der Aufgaben
-- Aufbau:
--    Hauptdatei:   
--        - Verweise auf die Problemklassen
--        - Allgemeine Erklaerungen
--      Problemklassendateien:
--          - Problembeschreibung
--          - Verweise auf die Aufgaben
--        Aufgabendateien: 
--            - Aufgabe als ASCII
--            - Aufgabe als GIF
--            - Loesung falls schon abgeschlossen
-- autor Georg Martius
-- mai99dgf@studserv.uni-leipzig.de

module Web.AlleAufgaben 
	(
		erzeugeIndex
    ) where

import Web.Html
import Web.Instanz
import Web.Konfiguration

import qualified Challenger
import qualified Instanzen
import qualified Beweise
import Problems

import System


makeIO :: a -> IO(a)
makeIO x = do
    return x

erzeugeIndex :: IO ExitCode
erzeugeIndex = do
	euzeugeHauptIndex problemListe
	system ("chmod -R o+r " ++ htmlWurzelverzeichnis ++ "*")
	

euzeugeHauptIndex :: [String] -> IO()
euzeugeHauptIndex  probs = do
    (inhaltHtml, ankerliste) <- erzeugeProblemBereiche  probs []
    let html = HTMLDokument "Challenger Hauptseite" 
    			(   [erzeugeAnker ankerliste] 
                ++ inhaltHtml
                )
    toFile (htmlWurzelverzeichnis ++ "index.html") html 
    
-- (1) ProblemListe
-- (2) Ankerliste
-- return  Html, Ankerliste
erzeugeProblemBereiche :: [String] -> [Anker] -> IO([HTMLElement],[Anker])
erzeugeProblemBereiche [] ankerliste = do 
		return ([TextPlain ""],ankerliste) 
erzeugeProblemBereiche (p:probs) ankerliste = do
    (aktuellesProblem, ankerl1) <- erzeugeProblem p ankerliste
    (restHtml, ankerl2) <- erzeugeProblemBereiche probs ankerl1
    return ( [aktuellesProblem] ++ restHtml, ankerl2 )  

erzeugeProblem :: String -> [Anker] -> IO(HTMLElement,[Anker])
erzeugeProblem prob ankerliste = do
    erlaeuterung <- problemBeschreibung prob
    instanzenliste <-  erzeugeInstanzenliste prob
    return ( Section (TextPlain prob)
    		 [
           SubSection (TextPlain "Erl&auml;uterung") 
             [ TextPlain erlaeuterung ]
         , SubSection (TextPlain "Beispiele") 
             [ (Link (prob ++ "/beispiele.html") "Zu den Beispielen")]
         , SubSection (TextPlain "Statistik") 
	           [(Link (statistikDatei prob) "Zur Statistik")]              
         , SubSection (TextPlain "Aufgaben")
            [instanzenliste]
         ]
           , ankerliste ++ [prob]
           )

-- | 
-- (1) Problem als String
erzeugeInstanzenliste :: String -> IO(HTMLElement)
erzeugeInstanzenliste p = do 
    iListe <- Instanzen.getAlleNachString p
    let instanzListe = Instanzen.nurKorrekte iListe
    return $ List 
    	( map (\ x -> Link 
        			(aufgabenDateiname p (Challenger.aufgabe(Instanzen.ident x))) 
                    (show (Challenger.aufgabe(Instanzen.ident x))) 
              ) instanzListe
        )   
        
