-- | Module Html bietet Funktionen zur Erstellung von HTML-Dokumenten
module Web.Html where

import qualified Datei as D

type Anker = String
-- | HTML-Element
-- ist entweder eine Sektion, SubSektion, Link, Tabelle, Image oder Text
data HTMLElement =
  HTMLDokument String [HTMLElement]
  | Image String String
  | Link String String
  | Section HTMLElement [HTMLElement]
  | SubSection HTMLElement [HTMLElement]
  | List [HTMLElement]
  | Tabelle [[HTMLElement]]
  | Text String
  | TextPlain String
  | TextVerbatim String


-- | toHTML wandelt die Datenstruktur HTMLElement in HTML-Text um
toHTML:: HTMLElement-> String
toHTML (HTMLDokument title elemente) =
  "<html>\n" ++
  "<head>\n" ++
  "  <title>" ++ title++ "</title>\n"++
  "  <link rel=\"stylesheet\" type=\"text/css\" href=\"/challenger/style.css\">\n" ++
  "</head>\n" ++
  "<body>\n" ++ concat (map toHTML elemente) ++ "\n</body>\n" ++
  "</html>\n"
toHTML (Link url name) =
  "\n<a href=\""++ url++ "\">" ++ name ++ "</a>\n"
toHTML (Image url name) =
  "\n<img src=\""++ url++ "\">" ++ name ++ "</a>\n"
toHTML (Section title body) =
  "\n<div class=\"section\">"
  ++ "<a name=\"" ++ (toHTML title) ++ "\"></a>"
  ++ (toHTML title)++ "</div>\n"++
  concat (map toHTML body)
toHTML (SubSection title body) =
  "\n<div class=\"subsection\">"++ (toHTML title)++ "</div>\n"++
  "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"transparent\">\n"++
  "<tr>\n"++
  "  <td class=\"transparent\" width=20><img src=\"spacer.gif\" width=20 height=1></td>\n"++
  "  <td class=\"transparent\">"++ concat (map toHTML body)++ "</td>"++
  "</tr>"++
  "</table>"
toHTML (Tabelle table) =
  "<table border=\"0\" cellspacing=\"1\" cellpadding=\"3\">\n"++
  (concat (map zeilen (map (map spalten) table)))++
  "</table>\n"
  where {
    zeilen (zeile::[String]) = "  <tr>"++ concat zeile++ "</tr>\n";
    spalten (zelle::HTMLElement) = "<td>"++ toHTML zelle ++ "&nbsp;" ++ "</td>"
  }
toHTML (List items) =
  "<ul>\n"++
  (concat (map listitems items))++
  "</ul>\n"
  where {
    listitems (item :: HTMLElement) = "<li>"++ toHTML item ++ "</li>"
  }
toHTML (Text a) =
  "\n<font class=\"text\">"++
  a++
  "</font>\n"
toHTML (TextPlain a) =
  a

toHTML (TextVerbatim a) =
  "\n<table class=\"haskellcode\"> <tr><td><pre>"++
  a ++
  "</pre></td></tr></table>\n"
	


-- | toFile schreibt ein HTML-Dokument in eine Datei
toFile:: D.Datei -> HTMLElement -> IO()
toFile file dokument =
   D.schreiben file (toHTML dokument)

---------------------------------------------------------------------------------

erzeugeAnker :: [Anker] -> HTMLElement
erzeugeAnker ankerliste = 
	Section (TextPlain "Inhalt" )
	  [List (map (\ x -> Link ("#" ++ x) x ) ankerliste)
      ]
                             
