-- | erzeugt die komplette Html Representation der Aufgaben als Index
-- Aufbau:
--    Hauptdatei:   
--        - Verweise auf die Problemklassen
--        - Allgemeine Erklaerungen
--      Problemklassendateien:
--          - Problembeschreibung
--          - Verweise auf die Aufgaben
--        Aufgabendateien: 
--            - Aufgabe als ASCII
--            - Aufgabe als GIF
--            - Loesung falls schon abgeschlossen
-- autor Georg Martius
-- mai99dgf@studserv.uni-leipzig.de

module Web.Index 
	(
		erzeugeIndex
    ) where

import Web.Html
import Web.Instanz
import Web.Konfiguration

import qualified Challenger
import qualified Instanzen
import qualified Beweise
import Problems
import qualified Datei as D

import System


makeIO :: a -> IO(a)
makeIO x = do
    return x

aufgabenIndex :: String -> D.Datei
aufgabenIndex prob =  
    D.Datei { D.pfad = [ htmlWurzelverzeichnis
                       , prob
                       ]
            , D.name = "index.html"
            , D.relativzahl = 1
            }
erzeugeIndex :: IO()
erzeugeIndex = do
  erzeugeHauptIndex problemListe
  erzeugeProblemIndex problemListe 
	

erzeugeHauptIndex :: [String] -> IO()
erzeugeHauptIndex probs = do
    let html = HTMLDokument "Challenger Hauptseite" 
    			[ Section (TextPlain "Problem Klassen") 
            [List 
    	        ( map 
                  (\ x -> Link (D.relativieren (aufgabenIndex x)) x )
                  probs
              )
            ]   
          ]
    toFile D.Datei { D.pfad = [htmlWurzelverzeichnis], D.name = "index.html"} html 
    
-- (1) ProblemListe
-- return  ()
erzeugeProblemIndex :: [String] -> IO()
erzeugeProblemIndex [] = return ()
erzeugeProblemIndex (p:probs) = do
    aktuellesProblem <- erzeugeProblem p
    toFile (aufgabenIndex p) aktuellesProblem 
    erzeugeProblemIndex probs

erzeugeProblem :: String -> IO(HTMLElement)
erzeugeProblem prob = do
    erlaeuterung <- problemBeschreibung prob
    instanzenliste <- erzeugeInstanzenliste prob
    return $ HTMLDokument ("Challenger - " ++ prob)
        [ Section (TextPlain prob)
    		 [ SubSection (TextPlain "Erl&auml;uterung") 
             [ TextPlain erlaeuterung ]
         , SubSection (TextPlain "Beispiel") 
             [ (Link ("Aufgabe.hs") "eine Aufgabe")
             , (Link ("Loesung.hs") "eine passende Lsung")
	     ]
         , SubSection (TextPlain "Statistik") 
	           [ (Link ("statistik.html") "Zur Statistik") ]            
         , SubSection (TextPlain "Aufgaben")
             [ instanzenliste ]
         ]
        ]        
        

-- | 
-- (1) Problem als String
erzeugeInstanzenliste :: String -> IO(HTMLElement)
erzeugeInstanzenliste p = do 
    iListe <- Instanzen.getAlleNachString p
    let instanzListe = Instanzen.nurKorrekte iListe
    return $ List 
    	( map (\ x -> Link 
        			(  D.relativieren $ aufgabenDateiname p (Challenger.aufgabe(Instanzen.ident x)))
              (show (Challenger.aufgabe(Instanzen.ident x))) 
            ) instanzListe
      )   
        

-- | Beschreibung des Problems als Html 
-- (1) Problem (als String)
problemBeschreibung :: String -> IO(String)
problemBeschreibung p = do
    catch 
          ( D.lesen $ getBeschreibungsDatei p)
          ( \ any -> do return "nicht verfuegbar" ) 

-- | Dateiname Beschreibungsdatei pro Problem 
-- (1) Problem (als String)
getBeschreibungsDatei :: String -> D.Datei
getBeschreibungsDatei p =
  D.Datei { D.pfad = [htmlWurzelverzeichnis, p] 
          , D.name = "Beschreibung.html"
	  , D.relativzahl = 2
	  }

