-- | erzeugt die Html Representation einer Aufgabe (Instanz)
--        Aufgabendateien: 
--            - Aufgabe als ASCII
--            - Aufgabe als GIF
--            - Loesung falls schon abgeschlossen
-- autor Georg Martius
-- mai99dgf@studserv.uni-leipzig.de

module Web.Instanz 
	(
  	  erzeugeInstanzDatei    
    , aufgabenDateiname
    ) where

import Web.Html
import Instanzen
import qualified Challenger as C
import Web.Konfiguration
import Problems
import qualified Datei as D

import System

aufgabenDateiname :: String -> Int -> D.Datei
aufgabenDateiname prob inst =  
    D.Datei { D.pfad = [ htmlWurzelverzeichnis
                       , prob
                       , stringMitNullen (show inst) 4
                       ]
            , D.name = "index.html"
            , D.relativzahl = 2
            }

aufgabenVisDateirumpf :: String -> Int -> D.Datei
aufgabenVisDateirumpf prob inst = 
    D.Datei { D.pfad = [ htmlWurzelverzeichnis
                       , prob
                       , stringMitNullen (show inst) 4
                       , "graphiken"
                       ]
            , D.name = "instanz_viz"
            , D.relativzahl = 3
            }

beweisVisDateirumpf :: String -> Int -> Int -> D.Datei
beweisVisDateirumpf prob inst id = 
    D.Datei { D.pfad = [ htmlWurzelverzeichnis
                       , prob
                       , stringMitNullen (show inst) 4
                       , "graphiken"
                       ]
            , D.name = "beweis_" ++ show id ++ "_viz"
            , D.relativzahl = 3
            }                        
                              
-- | erzeugt die Html-Datei fuer eine Instanz
-- (1) Einsendung (Aufgabe)
-- (2) Autor
-- (3) Soll der Beweis angegeben werden?
-- return 
erzeugeInstanzDatei :: C.Problem p i b => C.Einsendung p i b -> C.Ident ->String -> Bool -> IO()
erzeugeInstanzDatei (e @ C.Aufgabe {}) ident autor mitBeweis = do  
   inhaltHtml <- erzeugeInhalt e ident autor mitBeweis
   let html = HTMLDokument 
    	      	( "Challenger - " ++ (show $ C.problem e) 
                	 ++ " - " ++ show ( C.aufgabe ident) 
              ) 
    	      	inhaltHtml
   toFile (aufgabenDateiname (show $ C.problem e) ( C.aufgabe ident)) html    


-- | erzeugt den Inhalt der instanzhtml-datei
-- (1) Einsendung (Aufgabe)
-- (2) Autor
-- (3) Soll der Beweis angegeben werden?
-- return HTML
erzeugeInhalt :: C.Problem p i b => C.Einsendung p i b -> C.Ident -> String ->  Bool -> IO ([HTMLElement])
erzeugeInhalt (e @ C.Aufgabe {}) ident  autor mitBeweis = do
    let ueberschrift =  (show $ C.problem e)  ++ " - " 
              	        ++ show ( C.aufgabe ident)
        aufgabeAscii = render $ toDoc (C.instanz e)
        ioLoesung = if mitBeweis 
        			then do erzeugeLoesung e ident
                    else do return (TextPlain "")
    loesung <- ioLoesung
    instanzvis <- ladeVisInstanz e ident
    return ([ Section (TextPlain ueberschrift)
    			     ([ SubSection (TextPlain "Autor") [ TextPlain autor ]
					  , SubSection (TextPlain "Aufgabe") 
                      	([
						   TextPlain "Als Haskell Struktur:<BR>\n"	
                      	 , TextVerbatim aufgabeAscii
                      	 ]
                         ++ instanzvis
                        )
                      ] 
                      
                     )
            , loesung
            ]
           )
           

erzeugeLoesung ::  C.Problem p i b => C.Einsendung p i b -> C.Ident -> IO(HTMLElement)
erzeugeLoesung e ident = do
     beweisvis <- ladeVisBeweis e ident 0
     return (
        SubSection (TextPlain "L&ouml;sung")
          ([
             TextPlain "Die L&ouml;sung des Autors dieser Aufgabe als Haskell Struktur:<BR>\n"
           , TextVerbatim (render $ toDoc (C.beweis e))               
           ] ++ beweisvis
           )
         )



ladeVisInstanz ::  C.Problem p i b => C.Einsendung p i b -> C.Ident  -> IO([HTMLElement])
ladeVisInstanz e ident = do
   let instdat =  aufgabenVisDateirumpf (show (C.problem e)) 
                                        (C.aufgabe ident)                                        
   D.erzeugeVerzeichnisse instdat
   (visdatei,typ,exitcode) <- C.getInstanz 
                    (C.problem e) (C.instanz e) (C.beweis e) 
                    (show instdat)
                    
   html	<- erzeugeVisHtml instdat typ exitcode
   return html

ladeVisBeweis ::  C.Problem p i b => C.Einsendung p i b -> C.Ident -> Int -> IO([HTMLElement])
ladeVisBeweis e ident id = do   
   let bewdat =  beweisVisDateirumpf (show (C.problem e)) 
                                     (C.aufgabe ident)
                                      id
   D.erzeugeVerzeichnisse bewdat                        
   (visdatei,typ,exitcode) <- C.getBeweis 
       								(C.problem e) (C.instanz e) (C.beweis e) 
					   				  (show bewdat)
   html	<- erzeugeVisHtml bewdat typ exitcode
   return html
   

-- | 
-- (1) Datei
-- (2) Typ
-- (3) Fehlercode                        
erzeugeVisHtml :: D.Datei -> String -> ExitCode -> IO([HTMLElement])                    
erzeugeVisHtml visdatei typ (ExitFailure num) = do
	return [TextPlain "Visualisierung fehlgeschlagen!<BR>"]
erzeugeVisHtml visdatei typ ExitSuccess = do	
  if typ == "html" 
    then do
        inhalt <- catch (readFile ((show visdatei) ++ "." ++ typ))  
                  (\any -> do return ("Datei nicht gefunden:" ++ (show visdatei) ++ "." ++ typ))
        return [ TextPlain "Die Aufgabe in einem schoeneren Format: <BR>"                  
               , TextPlain inhalt   
               ]
    else do
        return [
		         TextPlain "Die Aufgabe als Bild: <BR>"                  
               , Image ((D.relativieren visdatei) ++ "." ++ typ) ""
               ]

vonAbsoluteToRelative :: String -> String
vonAbsoluteToRelative datei = drop (length htmlWurzelverzeichnis) datei

-- | String wird auf n Zeichen aufgeblaeht (wenn laenger als n dann nichts)
-- (1) String, der aufgeblaeht werden soll
-- (2) Anzahl der Ziffern
-- return neuer String
stringMitNullen :: String -> Int -> String
stringMitNullen s len = 
    if length s < len 
        then
          (replicate (len - length s) '0') ++ s
        else   
          s

