module Web.Statistik 
  (
    problemStatistik
  )where

import Web.Html
import Web.StatistikKonfiguration
import Web.Konfiguration

import Challenger
import qualified Beweise as B
import qualified Instanzen as I

import Problems

import Locale
import Time
import Zeit

-- | Berechnung der Statitik fr ein Problem
-- (1) Problem als String
-- return schreibt eine HTML-Datei mit der Problem-Statistik
problemStatistik:: String-> IO ()
problemStatistik problem = do
  -- alle Instanzen des Problems ermitteln
  instanzen_list <- I.getAlleNachString problem
  let
    korrekte_instanzen = I.nurKorrekte instanzen_list
  -- fr jede Instanz eine Statistik aus den Beweisen erstellen
  instanzStatistiken <- sequence $ map (instanzStatistik problem) korrekte_instanzen
  let
    statistik= SubSection (TextPlain (problem)) [
                 Tabelle (
                   [ (TextPlain ""):(TextPlain "Instanz"):[TextPlain (x ++". Beweis") | x<-([a:[]| a<-['1'..'9']] ++ ["10"])]]
                   ++ [tabellen_zeile | (_, tabellen_zeile)<- instanzStatistiken]
                 )
               ]
    top_list= joinTopLists [top | (top, _)<- instanzStatistiken]
  toFile (statistikDatei problem)
    (HTMLDokument ("Autotool-Statistik: "++ problem) 
      ([Section (TextPlain ("Autotool-Statistik: "++ problem)) [statistik]]++ 
       [TextPlain "<br>"]++ -- nur als Platzhalter
       [SubSection (TextPlain "TopList") [topListToTabelle top_list]])

    )

-- | Berechnung der Statistik fr eine InstanzListe
-- (1) Problem als String
-- (2) Instanz (fr die eine Statistik erstellt werden soll)
-- return eine Punkte-Liste und ein Zeile einer Tabelle mit der Instanzstatistik
instanzStatistik:: String->I.Instanz-> IO ([(String,Int)],[HTMLElement])
instanzStatistik problem instanz = do
   -- die Statistik zu der Instanz erstellen (nur korrekte Beweise)
   beweise_list <- B.getAlleNachString problem (I.ident instanz)
   let
     -- nur die erste korrekte Einsendung eines Autors zhlt
     beweise = take 11 (B.nurKorrekte1ProAutor beweise_list)
     -- die Punkteverteilung berechnen -> TopListe erstellen
     punkte  = berechnePunktVerteilung (map B.datum beweise)
     top_list= verknuepfeListen (\x y-> (x,y)) (map B.autor beweise) punkte
     -- die Punktverteilung in HTML wandeln
     datum x = formatCalendarTime defaultTimeLocale "%d.%m %H:%M" (B.datum x)
     daten x = (B.autor x)++ "<br>" ++ (datum x)++ "<br>"
     liste   = (verknuepfeListen (++) [daten x| x<- beweise] (map show punkte))
     tabellen_zeile = liste++ ((mehrfach (11- length liste) ("":))[])
   return (top_list, (TextPlain "Autor:<br>Datum:<br>Punkte:"):[TextPlain x | x<- tabellen_zeile])

-- | zwei Punkte-Listen addieren/ zusammenfhren
-- (1) Punkte-Liste aus Paaren (Autor, Punkte)
-- (2) Punkte-Liste aus Paaren (Autor, Punkte)
-- return Punkte-Liste mit addierten Punktzahlen
join2TopLists:: [(String,Int)]->[(String,Int)]-> [(String,Int)]
join2TopLists [] lb = lb
join2TopLists ((aa, pa):la) lb = 
  let lb_aa= [(a,p) | (a,p)<-lb, a==aa] in
  if (length lb_aa) > 0
    then let (ab, pb)= head lb_aa in (aa, pa+pb):(join2TopLists la [x| x<-lb, notElem x lb_aa])
    else (aa, pa):(join2TopLists la lb)

-- | mehrere Punkte-Listen addieren/ zusammenfhren
-- (1) eine Menge von Punkte-Listen
-- return Punkte-Liste mit addierten Punktzahlen
joinTopLists:: [[(String,Int)]]-> [(String,Int)]
joinTopLists [] = []
joinTopLists (a:[]) = a
joinTopLists (a:l) = join2TopLists a (joinTopLists l)

-- | sortiert eine Punkte-Liste nach Punkten (naives Quicksort)
-- (1) Punkte-Liste aus Paaren (Autor, Punkte)
-- return sortierte Punkte-Liste
sortTopList:: [(String,Int)]->[(String,Int)]
sortTopList [] = []
sortTopList ((a,p):list) =
  (sortTopList [(ax,px)|(ax,px)<-list,px<=p])++
  [(a,p)]++
  (sortTopList [(ax,px)|(ax,px)<-list,px>p])

-- | wandelt eine TopListe in eine HTML-Tabelle
-- (1) Punkte-Liste aus Paaren (Autor, Punkte)
-- return HTMLTabelle mit den 10 Topscorern
topListToTabelle:: [(String,Int)]->HTMLElement
topListToTabelle list =
  (Tabelle ([ [TextPlain "Matr.Nr.:",TextPlain "Punkte"] ]++ 
            [ [TextPlain a, TextPlain (show p)] | (a,p)<-(take 11 (reverse (sortTopList list)))]))
