module Web.StatistikKonfiguration where 

import Time
import Zeit

-- | fr die Konfiguration 
punktePro_Aufgabe = 80:: Int 
punktePro_Einsendung = 20:: Int -- maximale Punkte fr eine Einsendung (Lsung/Aufgabe) 
 
-- wert in Prozent [0..100] 
wert_Ungeloest = 0:: Int -- eine bisher ungelste Aufgabe hat keinen Wert 
wert_SteigerungProLoesung = 15:: Int -- mehr Lsungen bringen mehr Wert (Aufgaben sollen nicht unlsbar sein) 
wert_einfache_Aufgabe = 10:: Int -- fr sehr einfache Aufgaben 
zeit_schwere_Aufgabe = 4* (24*60*60):: Int -- durchschnittliche Lsungszeit fr eine schwere Aufgabe 
wert_schwere_Aufgabe = 100:: Int -- fr anspruchsvolle Aufgaben 
 
-- | berechnet die Punktverteilung fr Einsendung und [Lsungen] 
-- Aufgabe: Punkte je nach Abstand der 1.Lsung und durchschnittlicher Lsungszeit 
-- Lsungen: Punkte je nach Zeit nach der Aufgabenstellung 
berechnePunktVerteilung:: [CalendarTime]-> [Int] 
berechnePunktVerteilung [] = []
berechnePunktVerteilung (aufgabe:loesungen) = 
  let 
  -- Berechnung der absoluten Abstnde der Lsugen zur Aufgabenstellung 
  zeiten= berechneZeiten aufgabe loesungen 
  -- anhand der Zeiten den Gesamtwert der Aufgabe berechnen 
  aufgabenwert= berechneAufgabenWert zeiten 
  -- der Aufgabensteller bekommt 1/5 des Gesamtwertes zugeschrieben 
  punkte_aufgabensteller= div aufgabenwert 5 
  -- die restlichen Lsungen bekommen Punkte fr den Platz+ Zeit 
  wertung_platz= [werte_Platz x | x<- [1..length loesungen]] 
  wertung_zeit= [werte_Zeit x | x<- zeiten] 
  -- fr den Platz und die Zeit werden jeweil 2/5 des Aufgabenwertes vergeben 
  normierung_wertung_platz x= div x (div (sum wertung_platz) (div (2*aufgabenwert) 5)) 
  normierung_wertung_zeit x= div x (div (sum wertung_zeit) (div (2*aufgabenwert) 5)) 
  punkte_loesung= verknuepfeListen (+) (map normierung_wertung_platz wertung_platz) (map normierung_wertung_zeit wertung_zeit) 
  in 
  (map (min punktePro_Aufgabe) (punkte_aufgabensteller:punkte_loesung)) 

-- | berechnet den Wert einer Aufgabe anhand der Zeitabstnde zwischen den Einsendungen 
berechneAufgabenWert:: [Int]-> Int 
berechneAufgabenWert [] = wert_Ungeloest -- die Aufgabe ist bisher ungelst 
berechneAufgabenWert zeiten = -- die Aufgabe wurde bereits gelst 
  let 
  -- Ma fr die Schwierigkeit des Problem 
  durchschnitt= gewichteterDurchschnitt berechneWichtung zeiten 
  -- Punkte fr die Aufgabe berechnen 
  wertLoesungen = min 100 (wert_Ungeloest+ (wert_SteigerungProLoesung* length zeiten)) 
  wertSchwierigkeit = min 100 (div (100* durchschnitt) zeit_schwere_Aufgabe) 
  in 
  div (punktePro_Aufgabe*((div (wertLoesungen*wertSchwierigkeit) 100)+ wertLoesungen+ wertSchwierigkeit)) 300 

-- | berechnet einen gewichteten Wert anhand der Position in der Kette 
berechneWichtung:: Int->Int->Int 
berechneWichtung n x =  
  div x (n+1) 
-- | Bewertung fr den Platz der Lsung 
werte_Platz:: Int-> Int 
werte_Platz platz = 
  div 100 (platz+1) 
-- | Bewertung der Zeit der Lsung 
werte_Zeit:: Int-> Int 
werte_Zeit sekunden = 
  div (335*24*60*60) sekunden -- 1.01^-sekunden wre vielleicht auch eine gute Verteilung? 

-- | berechnet einen gewichteten Durchschnitt 
gewichteterDurchschnitt:: (Int->Int->Int)->[Int]-> Int 
gewichteterDurchschnitt f liste = 
  div (100*(summeMitWichtung f liste)) (summeMitWichtung f (mehrfach (length liste) (100:) [])) 
-- | berechnet eine gewichtete Summe 
summeMitWichtung:: (Int->Int->Int)->[Int]-> Int 
summeMitWichtung f liste =  
  sum (position_map f liste)

-- | wie map, aber mir zustzlichem Argument der Position in der Liste 
position_map:: (Int->a->a)->[a]->[a] 
position_map _ [] = [] 
position_map f liste =  
  let pos= (length liste)-1 in (position_map f (take pos liste))++ [f pos (liste!!pos)] 

-- | berechnet die Zeit von Aufgabenstellung bis zu jeder Lsung in Sekunden 
berechneZeiten:: CalendarTime->[CalendarTime]->[Int] 
berechneZeiten _ [] = [] 
berechneZeiten aufgabe (loesung:rest) =  
  abs(sekundenDifferenz loesung aufgabe):(berechneZeiten aufgabe rest) 

-- | fhrt eine Funktion mehrfach hintereinander aus (gibt es hierfr eine Haskell Funktion?) 
mehrfach:: Int->(a->a)->a->a 
mehrfach n funktion ausdruck 
  | n==1 = funktion ausdruck 
  | n>1  = funktion (mehrfach (n-1) funktion ausdruck) 

-- | komponentenweise Addition von zwei Listen 
verknuepfeListen:: (a->b->c)->[a]->[b]->[c] 
verknuepfeListen f _ [] = [] 
verknuepfeListen f [] _ = [] 
verknuepfeListen f (a:la) (b:lb) = (f a b):(verknuepfeListen f la lb) 



