module Web.Statistik where


import Web.Html
import Web.StatistikKonfiguration
import Web.Konfiguration

import Challenger
import qualified Beweise as B
import qualified Instanzen as I
import Problems

import Locale
import Time
import Zeit


-- Liste der Probleme muss noch extern importiert werden
-- | Berechnung der Statistik
main:: IO()
main = do
  -- fr alle Probleme eine Statistik erstellen
  (top_list, statistiken)<- problemStatistik problemListe
  toFile (htmlWurzelverzeichnis ++ "statistik.html")
    (HTMLDokument "Autotool-Statistik" 
      ([Section (TextPlain "Autotool-Statistik") statistiken]++ 
       [TextPlain "<br>"]++ -- nur als Platzhalter
       [Section (TextPlain "TopList") [topListToTabelle top_list]])
    )

-- | Berechnung der Statitik fr eine ProblemListe
-- (1) ProblemListe als Strings
problemStatistik:: [String]-> IO ([(String,Int)],[HTMLElement])
problemStatistik [] = do 
		return ([],[])
problemStatistik (problem:liste) = do
  -- ProblemStatistiken der restlichen Liste
  (top_list, problem_statistiken) <- problemStatistik liste
  -- die Statistik zu problem erstellen
  instanzen_list <- I.getAlleNachString problem
  let
  	korrekte_instanzen = I.nurKorrekte instanzen_list
  -- fr jede fr die Instanzliste eine Statistik aus den Beweisen erstellen
  (new_top_list, instanz_statistiken) <- instanzStatistik problem korrekte_instanzen
  return 
      (joinTopList top_list new_top_list,
       ((SubSection (TextPlain (show problem)) 
      	 [
          Tabelle (
        	 [ Text "Instanz": [Text (x ++". Beweis") | x<-([a:[]| a<-['1'..'9']] ++ ["10"])]]
             ++ instanz_statistiken )
         ]
        ):problem_statistiken)
       )

-- | Berechnung der Statistik fr eine InstanzListe
instanzStatistik:: String -> [I.Instanz]-> IO ([(String,Int)],[[HTMLElement]])
instanzStatistik _ [] = do 
	return ([],[])
instanzStatistik problem (instanz:liste) = do
   -- InstanzStatistiken der restlichen Liste
   (top_list, instanz_statistiken) <- instanzStatistik problem liste
   -- die Statistik zu der Instanz erstellen (nur korrekte Beweise)
   beweise_list <- B.getAlleNachString problem (I.ident instanz)
   let
    korrekte_beweise = B.nurKorrekte beweise_list
    -- nur die erste korrekte Einsendung eines Autors zhlt (falls mehrfach eingesendet wurde)
    beweise = take 11 [ beweis | (l,beweis,r) <- mysplit korrekte_beweise, notElem (B.autor beweis) (map B.autor l)]
    -- die Punkteverteilung berechnen -> neue TopListe erstellen
    punkte  = berechnePunktVerteilung (map B.datum beweise)
    new_top_list = joinTopList (verknuepfeListen (\x y-> (x,y)) (map B.autor beweise) punkte) top_list
    -- die Punktverteilung in HTML wandeln
    datum x = formatCalendarTime defaultTimeLocale "%d.%m %H:%M" (B.datum x)
    daten x =  (B.autor x)++ "<br>" ++ (datum x)++ "<br>"
    liste = (verknuepfeListen (++) [daten x| x<- beweise] (map show punkte))
    tabellen_zeile = liste++ ((mehrfach (11- length liste) ("":))[])

   return (new_top_list, ([Text x | x<- tabellen_zeile]:instanz_statistiken))

-- | alle Aufsplittungen der Liste berechnen (in linke und rechte Teile)
mysplit:: [a]->[([a],a,[a])]
mysplit [] = []
mysplit (element:liste) = [([],element,liste)]++[(element:a,b,c) | (a,b,c)<- mysplit liste]

-- | die Punkte aus zwei Listen zusammenfhren
joinTopList:: [(String,Int)]->[(String,Int)]->[(String,Int)]
joinTopList [] lb = lb
joinTopList ((aa, pa):la) lb = 
  let lb_aa= [(a,p) | (a,p)<-lb, a==aa] in
  if (length lb_aa) > 0
    then let (ab, pb)= head lb_aa in (aa, pa+pb):(joinTopList la [x| x<-lb, notElem x lb_aa])
    else (aa, pa):(joinTopList la lb)

sortTopList:: [(String,Int)]->[(String,Int)]
sortTopList [] = []
sortTopList ((a,p):list) =
  (sortTopList [(ax,px)|(ax,px)<-list,px<=p])++
  [(a,p)]++
  (sortTopList [(ax,px)|(ax,px)<-list,px>p])

topListToTabelle:: [(String,Int)]->HTMLElement
topListToTabelle list =
  (Tabelle ([ [Text "Matr.Nr.:",Text "Punkte"] ]++ 
            [ [Text a, Text (show p)] | (a,p)<-(take 11 (reverse (sortTopList list)))]))

