-- $Header: /home/joe/scratch/challenger/autotool/fa/NFA_Type.hs,v 1.2 2002/06/17 00:46:36 joe Exp $

module NFA_Type

where

import FAU
import Size

import ReadSet
import ReadFM

import ToDoc

import Monad (guard)


data NFA s = 
     NFA { states :: Set s
	 , starts :: Set s
	 , finals :: Set s
	 , trans  :: FiniteMap (s, Char) (Set s)
	 } deriving (Read)

--instance (Read s) => Read (NFA s) where readsPrec _ = read

instance Size (NFA s) where size = cardinality . states

instance (ToDoc s, ToDoc (Set s)) => ToDoc (NFA s) where
    toDoc a = 
	 let pstates = text "states" <+> equals <+> toDoc (states a)
	     pstarts = text "starts" <+> equals <+> toDoc (starts a)
	     pfinals = text "finals" <+> equals <+> toDoc (finals a)
	     ptrans  = text "trans " <+> equals <+> toDoc (trans  a)

	 in      text "NFA" 
	     <+> braces ( fsep $ punctuate comma 
				    [ pstates, pstarts, pfinals, ptrans] )

instance ToDoc (NFA s) => Show (NFA s) where show = render . toDoc

---------------------------------------------------------------------

collect :: Ord s => [(s, Char, s)] -> FiniteMap (s, Char) (Set s)
collect pxqs = addListToFM_C union emptyFM $ do
    (p, x, q) <- pxqs
    return ((p, x), unitSet q)

unCollect :: FiniteMap (s, Char) (Set s) -> [(s, Char, s)]
unCollect fm = do
    ((p,c), qs) <- fmToList fm
    q <- setToList qs
    return (p, c, q)

---------------------------------------------------------------------





---------------------------------------------------------------------


lstates = setToList . states
lstarts = setToList . starts
ltrans  = fmToList  . trans
lfinals = setToList . finals

letters :: NFA s -> [ Char ]
letters a = setToList $ mkSet
	      $ do ((p, c), qs) <- ltrans a; return c

---------------------------------------------------------------------

ceps :: Ord s => NFA s -> Bool
-- accepts epsilon?
ceps a = not $ isEmptySet $ intersect (starts a) (finals a)

---------------------------------------------------------------------

-- polynomialzeit

reaches :: Ord s => NFA s -> String -> Set s
reaches a w = rch w (starts a)
    where rch [] qs = qs
	  rch (c : cs) qs = 
	      let ps = mkSet $ do
		     q <- setToList qs
		     setToList $ lookupWithDefaultFM (trans a) emptySet (q, c)
	      in  rch cs ps

is_accepted_by :: Ord s => String -> NFA s -> Bool
is_accepted_by w a = 
    not $ isEmptySet $ intersect (reaches a w) (finals a)	  

---------------------------------------------------------------------

amap :: (Ord s, Ord t) => (s -> t) -> (NFA s -> NFA t)
amap f n =
    NFA { states = smap f . states $ n
	, starts = smap f $ starts n
	, finals = smap f . finals $ n
	, trans = addListToFM_C union emptyFM $ do
	      ((p, c), qs) <- fmToList $ trans n
	      return ((f p, c), smap f qs)
	}

afilter :: Ord s => (s -> Bool) -> NFA s -> NFA s
afilter f a =
    	 a { states = sfilter f $ states a
	   , finals = sfilter f $ finals a
	   , starts = sfilter f $ starts a
	   , trans = listToFM $ do 
	        ((p, c), qs) <- ltrans a
	        guard $ f p
		let qs' = sfilter f qs
		guard $ not $ isEmptySet qs'
	        return ((p, c), qs')
	   } 

---------------------------------------------------------------------



