-- $Header: /home/joe/scratch/challenger/autotool/fa/Ops.hs,v 1.1.1.1 2002/05/24 10:46:47 challenger Exp $

module Ops where

-- $Log: Ops.hs,v $
-- Revision 1.1.1.1  2002/05/24 10:46:47  challenger
-- start
--
-- Revision 1.5  2001/11/19 08:04:22  autotool
-- fehler in Minus.complement behoben
--
-- Revision 1.4  2001/11/16 11:44:03  autotool
-- fr NFAcommon: benutze neue Funktion Minus.minus_det
-- zum vergleichen, spare das deterministisch-und-minimieren.
--

import NFA_Type
import Trim
import Basic

import Monad (guard)

import qualified FAU

import FiniteMap
import qualified Set

cross :: (Ord s, Ord t) => NFA s -> NFA t -> NFA (s, t)
cross a b =
    NFA { states = FAU.cross (states a) (states b)
	, starts = FAU.cross (starts a) (starts b)
	, finals = error "cross.finals unspecified"
	, trans = addListToFM_C Set.union emptyFM $ do 
	     ((pa, ca), qas) <- ltrans a
	     ((pb, cb), qbs) <- ltrans b
	     guard $ ca == cb
	     return (((pa, pb), ca), FAU.cross qas qbs)	
	}
  

union :: (Ord s, Ord t) => NFA s -> NFA t -> NFA (Either s t)
union a b = 
    let a' = amap Left a
	b' = amap Right b
    in	NFA { states = states a' `Set.union` states b'
	    , starts = starts a' `Set.union` starts b'
	    , finals = finals a' `Set.union` finals b'
	    , trans = plusFM_C (error "union") (trans a') (trans b')
	    }

intersection :: (Ord s, Ord t) => NFA s -> NFA t -> NFA (s, t)
intersection a b = 
    let c = cross a b
	ok (p, q) =  Set.elementOf p (finals a)	
		  && Set.elementOf q (finals b)	
    in	trim $ c { finals = FAU.sfilter ok $ states c }


punkt :: Ord s => s -> NFA s -> NFA s
-- addiere neuen Startzustand,
-- in den keine Pfeile hineinfhren
punkt t a = 
    a { states = Set.union (states a) (Set.unitSet t)
      , starts = Set.unitSet t
      , finals = if not $ Set.isEmptySet $ Set.intersect (starts a) (finals a) 
		 then Set.union (finals a) (Set.unitSet t)
		 else            finals a
      , trans = addListToFM_C Set.union (trans a)
	      $ do ((p, c), qs) <- ltrans a
		   guard $ Set.elementOf p (starts a)
		   return ((t, c), qs)
      }

neu :: NFA Int -> Int
neu a = 1 + maximum (0 : Set.setToList (states a))


dot :: NFA Int -> NFA Int -> NFA Int
dot a b =
    let n = neu a
	b' = amap (+ n) b
    in	trim $
	NFA { states = Set.union (states a) (states b')
	    , starts = if ceps a
		       then Set.union (starts a) (starts b')
		       else starts a
	    , finals = if ceps b' 
		       then Set.union (finals a) (finals b')
		       else finals b'
	    , trans = addListToFM_C Set.union
	          (plusFM_C (error "dot") (trans a) (trans b'))
		  $ do ((p, c), qs) <- ltrans b'
		       guard $ Set.elementOf p $ starts b'
		       f <- lfinals a
		       return ((f, c), qs)
	    }

power :: Integer -> NFA Int -> NFA Int
power e a = foldr dot epsilon $ take (fromInteger e) $ repeat a

plus :: NFA Int -> NFA Int
plus a = 
    let n = neu a
	b = punkt n a
    in	trim $ b { trans = addListToFM_C Set.union (trans b)
		  $ do ((p, c), qs) <- ltrans b
		       f <- lfinals b
		       guard $ Set.elementOf p $ starts b
		       return ((f, c), qs)
	  }

star :: NFA Int -> NFA Int
star a =
    let b = plus a
    in	b { finals = Set.union (finals b) (starts b) }


