-- $Header: /home/joe/scratch/challenger/autotool/fa/Restrict.hs,v 1.1.1.1 2002/05/24 10:46:47 challenger Exp $

module Restrict where

import NFA
import Set
import FiniteMap
import Size
import FAU
import ToDoc
import Monad (guard)

restrict_states
   :: (ToDoc a, ToDoc [a], Ord a) 
   => NFA a -> Either String String
restrict_states a = 
    let forbidden q = not ( q `elementOf` states a )
	nostarts  = sfilter forbidden $ starts a
	nofinals  = sfilter forbidden $ finals a
	notrans   = do
	    ((p, x), qs) <- ltrans a
	    q <- setToList qs
	    guard $ forbidden p || forbidden q
	    return (p, x, q)
	msgs =  [ text "Diese Startzustnde sind nicht deklariert: " 
		     <> toDoc nostarts | nonempty nostarts ]
	     ++ [ text "Diese akzeptierenden Zustnde sind nicht deklariert:"
		     <> toDoc nofinals | nonempty nofinals ]
	     ++ [ text "Diese bergangsregeln benutzen nicht deklarierte Zustnde:"
		     <> toDoc notrans | not (null notrans) ]
    in	
	if null msgs
	then Right "OK: Alle benutzten Zustnde wurden auch deklariert."
	else Left $ render $ hsep msgs



restrict_alpha
   :: (ToDoc a, Ord a) 
   => Set Char -> NFA a -> Either String String
restrict_alpha alpha a = 
    let
	noletters = do
	    ((p, x), qs) <- ltrans a
	    q <- setToList qs
	    guard $ not (x `elementOf` alpha)
	    return (p, x, q)
	msg = text "Diese bergangsregeln benutzen nicht erlaubte Buchstaben:"
		     <> toDoc noletters

    in	if null noletters 
	then Right "OK: Die bergangsregeln benutzen nur erlaubte Buchstaben."
	else Left $ render msg

restrict_size
   :: (ToDoc a, Ord a) 
   => Int -> NFA a -> Either String String
restrict_size s a = 
    if size a <= s
    then Right "OK: Der Automat hat nicht mehr Zustnde, als in dieser Aufgabe erlaubt sind."
    else Left $ unlines
	      [ "Dieser Automat hat " ++ show (size a) ++ " Zustnde,"
	      , "erlaubt sind in dieser Aufgabe aber nur " ++ show s ++ "."
	      ]
