module Rewrite 

( rewrites 
)

where

import NFA_Type
import qualified Ops
import Normalize
import Minimize
import Trim

import Set
import FiniteMap

rewrites :: Ord t => [(String, String)] -> NFA t -> NFA Int
-- alle Ein-Schritt-Nachfolger bzgl. eines Wort-Ersetzungs-Systems
rewrites rs a  = foldr1 ( \ x y -> normalize $ Ops.union x y ) 
	       $ map (normalize . rewrite a) rs

-- benutzt im Automaten, der von "vor regel-match"
-- zu "regel-match" zu "nach regel-match" wechselt
data State t = Pre t | Mid (Int, Int) | Post t
     deriving (Eq, Ord, Read, Show)

rewrite :: Ord t =>  NFA t -> (String, String) -> NFA (State t)
-- funkt nur fr length rhs > 0 !
rewrite (a :: NFA t) (lhs, rhs) = 
    let 
	pre = amap Pre a :: NFA (State t)
	post = amap Post a :: NFA (State t)
	both = plusFM_C (error "Rewrite.rewrite") (trans pre) (trans post)

	extra_rules :: [(State t, Char, State t)]
	extra_rules =  do
	    (k, (p, q)) <- zip [0..] $ pairs a lhs
	    let qs :: [ State t ]
		qs =    [ Pre p ] 
		     ++ [ Mid (k, i) | i <- [ 1 .. length rhs - 1 ] ]
		     ++ [ Post q ]
	    (c, (x,y)) <- zip rhs $ zip qs $ tail qs
	    return (x :: State t, c, y :: State t)
	extra_states = mkSet $ do
	    (x, c, y) <- extra_rules
	    [x, y]
    in	trim $ NFA 
	     { states = unionManySets [ states pre, extra_states, states post ]
	     , starts = starts pre
	     , finals = finals post
	     , trans = plusFM_C union both $ collect extra_rules 
	     }





pairs :: Ord t => NFA t -> String -> [(t, t)]
-- liste von paaren von zustnden (q1, q2),
-- so da es in a einen mit w markierten pfad von q1 nach q2 gibt
pairs a w = do
    p <- setToList $ states a
    q <- setToList $ reaches (a { starts = mkSet [p] }) w
    return (p, q)





-- SO GEHTS NICHT

single :: Set Char -> String -> NFA Int
-- erzeugt automaten, der Sigma^* w Sigma^* akzeptiert,
-- mit zustnden 0, 1, .. |w|
single sigma w =
    let n = length w
    in	NFA { states = mkSet [ 0 .. n ]
	    , starts = mkSet [ 0 ]
	    , finals = mkSet [ n ]
	    , trans  = addListToFM_C union emptyFM 
		     $	[ (( i, w !! i), mkSet [i+1] ) 
			| i <- [ 0 .. n - 1 ] ]
		     ++ [ ((i, c), mkSet [i]) 
			| i <- [0,n], c <- setToList sigma ]
		     
	    }

				





