module Rose where


import List (intersperse)

data Rose a = Node { item :: a
		   , children :: [ Rose a ] 
		   } 
     deriving Show

instance Functor Rose where
    fmap f (Node x cs) = Node (f x) (map (fmap f) cs)

rfold :: (a -> [b] -> b) -> (Rose a -> b)
rfold f t = f (item t) $ map (rfold f) $ children t

----------------------------------------------------------

levels :: Rose a -> [ [a] ]
levels n = schichten [ n ]

schichten [] = []
schichten cs = map item cs 
	     : schichten ( concat (map children cs) )

bfs :: Rose a -> [a]
bfs = concat . levels

dfs :: Rose a -> [a]
dfs (Node x cs) = x : concat (map dfs cs)

prune :: (a -> Bool) -> Rose a -> Rose a
prune f (Node x cs) = 
    Node x $ if f x then map (prune f) cs else [] 

trim :: Int -> Rose a -> Rose a
trim 0 t = t { children = [] }
trim d t = t { children = map (trim $ d-1) $ children t }

compact :: Rose a -> Rose a
compact t = case children t of
	[ c ] -> compact c
	_ -> t { children = map compact $ children t }


    


