-- $Header: /home/joe/scratch/challenger/autotool/fa/Shortest.hs,v 1.1.1.1 2002/05/24 10:46:47 challenger Exp $

-- $Log: Shortest.hs,v $
-- Revision 1.1.1.1  2002/05/24 10:46:47  challenger
-- start
--
-- Revision 1.4  2001/12/01 14:47:33  autotool
-- prekompilierte grader
--
-- Revision 1.3  2001/11/08 14:54:27  autotool
-- testet auch auf endlichkeit der sprache
--
-- Revision 1.2  2001/11/08 14:03:11  autotool
-- Shortest.hs : test, ob akzeptierte sprache leer
--    damit wir dann keine bottom-liste ausgeben, sondern []
--

module Shortest where

import NFA_Type
import Trim
import Set
import FAU
import FiniteMap

import Monad (guard)
import List (nub)

type State s  = (String, s)

succs :: Ord s => NFA s -> State s -> Set ( State s )
succs a (w, p) = mkSet $ do
    c <- letters a
    q <- setToList $ lookupWithDefaultFM (trans a) emptySet (p, c)
    return (w ++ [c], q)

computations :: Ord s => NFA s -> [ Set ( State s ) ]
computations a = comps ( smap ( \ p -> ( "", p ) ) $ starts a ) where
    comps todo = todo :
	  let next = unionManySets $ setToList $ smap (succs a) todo
	  in  if isEmptySet next then [] else comps next

accepted :: Ord s => NFA s -> [ String ]
accepted a = 
    let ws = do
	   wqs <- computations a
	   return $ nub $ do
	       ( w, q ) <- setToList wqs
	       guard $ q `elementOf` finals a
	       return w
        n = cardinality $ states a
	(short, long) = splitAt n ws
	(mid, rest)   = splitAt n long
	
	here  = concat short
	there = concat mid
	after = concat rest

    in	here ++ if null here then [] 
		else there ++ if null there then []
			      else after

shortest :: Ord s => NFA s -> [ String ]
shortest a = 
    let b = trim a
    in	case accepted b of
	     [] -> []
	     rest @ ( w : _ ) -> takeWhile (\ u -> length u == length w) rest

------------------------------------------------------------------------------

reachables :: Ord s => NFA s -> String -> Set s
reachables a w = reach ( starts a ) w
    where reach ps [] = ps
	  reach ps ( c : cs ) = 
	      let qs = mkSet $ do
		     p <- setToList ps
		     setToList $ lookupWithDefaultFM (trans a) emptySet (p, c)
	      in  reach qs cs

is_accepted :: Ord s => NFA s -> String -> Bool
is_accepted a w = 
    not $ isEmptySet $ intersect ( reachables a w ) ( finals a )
