module Trans 

( classes

)

where

import NFA
import FAU

import Set

import qualified Basic
import qualified Ops
import Normalize
import Minimize
import Trim
import Hull

import Array
import Monad (guard)


type Trans s = Array (s, s) Bool

---------------------------------------------------------------------

dot :: (Enum s, Ix s) => Trans s -> Trans s -> Trans s
dot a b = 
    if bounds a /= bounds b 
    then error "Trans.dot"
    else array (bounds a) $ do
	let ((lo,_),(hi,_)) = bounds a
	(x, z) <- indices a
	let j = or $ do y <- [ lo .. hi ]; return $ a!(x,y) && b!(y,z)
	return ((x, z), j)

---------------------------------------------------------------------

mkone :: (Enum s, Ix s) => NFA s ->  Trans s 
mkone a = 
    let qs = lstates a
	(p, q) = (minimum qs, maximum qs)
	bnd = ((p,p),(q,q))
    in	accumArray ( \ o n -> n ) False bnd $ do
	p <- lstates a
	return ((p, p), True)

mkletters :: (Enum s, Ix s) => [ Char ] -> NFA s -> [ (Char, Trans s) ]
mkletters alpha a = do
    let qs = lstates a
    let (p, q) = (minimum qs, maximum qs)
    let bnd = ((p,p),(q,q))
    c <- alpha
    return ( c
	   , accumArray ( \ o n -> n ) False bnd $ do
		((p, d), qs) <- ltrans a
		guard $ c == d
		q <- setToList qs
		return $ ((p, q), True)
	   )

---------------------------------------------------------------------

classes :: (Enum s, Ix s) => [Char ] -> NFA s -> [ NFA Int ]
classes alpha a = do
    let tr = listToFM $ mkletters alpha a 
    let next x c = x `dot` lookupWithDefaultFM tr (error "Trans") c
    let start = mkone a
    let tab = listToFM $ hull alpha next start
    let b = NFA { states = mkSet $ map fst $ keysFM tab
		, starts = unitSet start
		, finals = undefined
		, trans  = mapFM ( \ (x, c) t -> unitSet t ) tab
		}
    final <- lstates b
    let c = b { finals = unitSet final }
    return $ minimize $ trim $ normalize $ c



