module Eval where

import Fun
import Trace
import FiniteMap
import Builtin

data Exp 
	= Zahl Integer
	| App Fun [ Exp ]
    deriving ( Eq, Ord, Show )


data Cache = Cache ( FiniteMap Exp Integer )
	deriving Show

evaluate :: Builtins -> Fun -> [ Integer ] -> ( [ String ] , Cache, Integer )
evaluate b f args = traced $ do
    put $ Cache emptyFM
    eval b $ App f $ map Zahl args

eval :: Builtins -> Exp -> Trace Cache Integer
eval b e = do
    trace $ "start eval " ++ show e
    x <- eval0 b e
    trace $ "wert eval " ++ show e ++ " = " ++ show x
    return x

eval0 :: Builtins -> Exp -> Trace Cache Integer
eval0 b e = 
    case e of
	Zahl n -> return $ n
	App (Zero a) args -> return 0
	_ -> cached_eval0 b e


cached_eval0 :: Builtins -> Exp -> Trace Cache Integer
cached_eval0 b e = do
    Cache fm <- get
    case lookupFM fm e of
	Just x -> do
	    trace $ "schon im Cache"
	    return x
	Nothing -> do
	    x <- really_eval0 b e
	    Cache fm <- get
	    put $ Cache $ addToFM_C (error "cached_eval") fm e x
	    return x


really_eval0 :: Builtins -> Exp -> Trace Cache Integer
really_eval0 b e = case e of

	App (Builtin a name) args -> do
	    let (arity, f) = lookupWithDefaultFM b 
		   (error $ "das Builtin `" ++ name ++ "' gibt es nicht." ) 
		   name
	    xs <- sequence $ map (eval b) args
	    return $ f xs

	App (Succ 1) [ arg ] -> do
	    x <- eval b arg
	    return $ x + 1

	App (Proj oben unten) args -> do
	    x <- eval b $ args !! (unten - 1)
	    return x

	App ( Sub a ( g : fs ) ) args -> do
	    eval b $ App g $ do 
		f <- fs 
		return $ App f args

	App f @ ( Ind a [ g, h ] ) args -> do
            let (xs, [y]) = splitAt (a - 1) args
	    yy <- eval b y
    	    if 0 == yy 
		then eval b $ App g xs 
		else do let prev = xs ++ [ Zahl $ yy - 1 ]
			eval b $ App h $ prev ++ [ App f prev ] 



