module Primzahlen where

import Builtin

import Logik (wahr)
import List (group)

primes = sieve ( 2 : [ 3, 5 .. ] )
    where sieve (x : ys) = x : sieve (filter ( \ y -> 0 < y `rem` x )  ys )

primfaktoren :: Integer -> [ (Integer, Int) ]
primfaktoren = compact . primf primes where
    primf (p : ps) n =
	if n == 1 then [ ]
        else if n < p * p then [ n ]
	else if 0 == n `rem` p
	then p : primf (p : ps) (n `div` p)
	else primf ps n
    compact fs = do g <- group fs; return (head g, length g)

rest = lift2 $ \ x y -> if 0 == y then 0 else x `rem` y 
teilbar = lift2 $ \ x y ->  wahr ( 0 == x && 0 == y 
		    || ( 0 < y && 0 == x `rem` y ) )
pr    = lift1 $ \ k ->   (0 : primes) !! fromInteger k 
prim  = lift1 $ \ x -> wahr $  x == (head $ dropWhile (< x) primes )
fak   = lift1 $ \ x -> foldr (*) 1 [ 1 .. x ] 

pick n x = 
      if 0 == x then ( 0, 0 )
      else let pfs = primfaktoren x
	   in  if n < 1 || n > fromInt ( length pfs ) then ( 0, 0 )
	       else pfs !! ( fromInteger n - 1 )

g     = lift2 $ \ n x ->           fst $ pick n x 
e     = lift2 $ \ n x -> fromInt $ snd $ pick n x 

primzahlen :: Builtins
primzahlen = builtins
	     [ ( "rest", rest) , ( "teilbar", teilbar ) 
	     , ( "pr", pr ) , ( "prim"  ,  prim ) , ( "fak", fak ) 
	     , ( "g", g ), ( "e", e )
	     ]


