module Typeof

( typeof
, check_arity 
)

where

import Fun
import Report
import Monad ( guard )

check_arity :: String -> Int -> Fun -> Report ()
check_arity name a f = do
    t <- typeof f
    when (a /= t) $ fail $ "die Funktion `" ++ name ++ "' soll "
		  ++ show a ++ "-stellig sein, ist aber "
		  ++ show t ++ "-stellig."
		  
    return ()



typeof :: Fun -> Report Int

typeof f @ (Builtin a name) = do
    when ( a < 0 ) $ fail
	$ "Es gibt keine negativ-stellige Funktion: " ++ show f
    return a

typeof f @ (Zero a) = do
    when ( a < 0 ) $ fail
	$ "Es gibt keine negativ-stellige Funktion: " ++ show f
    return a

typeof f @ (Succ a) = do
    when ( a /= 1 ) $ fail $ "S mu einstellig sein: " ++ show f
    return 1

typeof f @ (Proj oben unten) = do
    when ( oben < 0 ) $ fail 
	$ "eine Projektion kann nicht negativ-stellig sein: " ++ show f
    when ( unten < 1 || oben < unten ) $ fail
	$ "eine " ++ show unten ++ "-te Komponente gibt es nicht: " ++ show f
    return oben

typeof f @ (Sub a fs) = do

    when ( a < 0 ) $ fail
  	$ "negativ-stellige Funktionen gibt es nicht: " ++ show f

    when ( null fs ) $ fail 
	$ "Sub braucht >= 1 Argument: " ++ show f

    -- typen der Argumente ausrechnen
    t : ts <- sequence $ map typeof fs

    -- anzahl der Argumente testen
    when ( t /= length ts ) $ fail 
	$ "Das erste Argument von Sub ist " ++ show t ++ "-stellig,\n"
	++ " aber es folgen nicht genau " ++ show t ++ " Funktionen: " 
	++ show f

    -- typen der argumente mit Ziel vergleichen
    let wrong = do (i, t) <- zip  [2..] ts; guard $ t /= a; return (i, t)
    when ( not $ null wrong ) $ fail
	$ let (i, t) = head wrong
	  in "es soll eine " ++ show a ++ "-stellige Funktion entstehen,\n"
	     ++ " aber das " ++ show i ++ "-te Argument von Sub "
	     ++ " ist " ++ show t ++ "-stellig: " ++ show f

    -- falls wir hier ankommen, ist alles OK
    return a


typeof f @ (Ind a fs) = do

    when ( a < 0 ) $ fail
  	$ "negativ-stellige Funktionen gibt es nicht: " ++ show f

    when ( length fs /= 2 ) $  fail
	$ "Ind bentigt genau zwei Argumente: " ++ show f

    [ t1, t2 ] <- sequence $ map typeof fs 

    when ( t1 /= a - 1 ) $ fail
	$ "Durch Ind soll eine " ++ show a ++ "-stellige Funktion entstehen,\n"
	++ " aber das erste Argument ist " ++ show t1 ++ "-stellig: "
	++ show f

    when ( t2 /= a + 1 ) $ fail
	$ "Durch Ind soll eine " ++ show a ++ "-stellige Funktion entstehen,\n"
	++ " aber das zweite Argument ist " ++ show t2 ++ "-stellig: "
	++ show f

    return a


typeof f @ (Min a fs) = do

    when ( a < 0 ) $ fail
  	$ "negativ-stellige Funktionen gibt es nicht: " ++ show f

    when ( length fs /= 1 ) $  fail
	$ "Min bentigt genau ein Argument: " ++ show f

    [ t1 ] <- sequence $ map typeof fs 

    when ( t1 /= a + 1 ) $ fail
	$ "Durch Min soll eine " ++ show a ++ "-stellige Funktion entstehen,\n"
	++ " aber das Argument ist " ++ show t1 ++ "-stellig: "
	++ show f

    return a
