module Reduziert 

( reduziert
, reduktion
)

where

import Monad (guard)
import Set
import FilterSet

import Grammatik
import Produktiv
import Erreichbar

einschrnkung :: Set Char -> Grammatik -> Grammatik
einschrnkung qs g =
  g { nichtterminale = filterSet (`elementOf` qs) $ nichtterminale g
    , regeln = mkSet $ do 
	          rule @ ( [lhs], rhs ) <- rules g
		  guard $ lhs `elementOf` qs
		  guard $ and $ do x <- rhs
				   guard $ x `elementOf` nichtterminale g
				   return $ x `elementOf` qs
		  return rule
    }

reduktion :: Grammatik -> Grammatik
reduktion g = let
    e = erreichbar g
    eg = einschrnkung e g
    p = produktiv eg
    pg = einschrnkung p eg
  in   pg

reduziert :: Grammatik -> Either String String
reduziert g = let
     a = nichtterminale g
     e = erreichbar g
     ne = minusSet a e
     eg = einschrnkung e g
     p = produktiv eg
     np = minusSet e p
     pg = einschrnkung p eg
  in 
     if not $ isEmptySet ne 
     then Left $ "Diese Variablen sind nicht erreichbar: " ++ show ne
     else if not $ isEmptySet np
     then Left $ "Diese Variablen sind nicht produktiv: " ++ show np
     else Right $ "Diese Grammatik ist reduziert."



