-- $Header: /home/joe/scratch/challenger/autotool/grammatik/Typen.hs,v 1.1.1.1 2002/05/24 10:46:47 challenger Exp $

module Typen

( typ0
, typ1
, typ2
, typ3


, muss

, monoton
, kontextsensitiv
, kontextfrei
, linear
, rechtslinear, linkslinear

, epsfrei
, kettenfrei
, chomsky
, greibach
)

where

import Monad (guard)
import List (partition)
import Set
import FilterSet

import Auswertung
import Wrong
import Grammatik
import Wort

---------------------------------------------------------------------------

nein0 msg = Left $ "Das ist keine Typ-0-Grammatik.\n" ++ msg

typ0 :: Grammatik -> Either String String
typ0 g = let 
    cs = filterSet ( `elementOf` (terminale g) ) 
       $ nichtterminale g 
    rs = filterSet ( \ (l, r) -> all ( `elementOf` (terminale g)) l ) 
       $ regeln g 
    us = filterSet ( \ (l, r) -> 
	    any ( not . ( `elementOf` ( terminale g `union` nichtterminale g ) ) ) 
		     (l ++ r)  ) 
       $ regeln g
 in if not $ isEmptySet cs
    then nein0 $ "Diese Zeichen sind Terminal UND Nicht-Terminal: " ++ show cs
    else if not $ isEmptySet rs
    then nein0 $  "Diese Regeln enthalten links kein Nichtterminal: " ++ show rs
    else if not $ isEmptySet us 
    then nein0 $ "Diese Regeln enthalten ein Zeichen, das weder Terminal noch Nichtterminal ist: " ++ show us

    else Right "Das ist eine Typ-0-Grammatik"
    
---------------------------------------------------------------------------

monoton :: Grammatik -> Either String String
monoton g = let 
    (eps, noeps) = partition ( null . snd ) $ rules g
    weps = mkSet $ filter ((/= [startsymbol g]) . fst) eps

    feps = mkSet $ do -- falls leeres wort erzeugt wird
	      guard $ not $ null $ eps
	      (l,r) <- rules g
	      guard $ startsymbol g `elem` r
	      return (l, r)
	      
    kurz = filter ( \ (l, r) -> length l > length r ) $ noeps

 in if not $ isEmptySet weps
    then Left $ unlines
	      [ "Regel(n) " ++ show weps
	      , "sind in monotoner Grammatik nicht gestattet."
	      , "Das leere Wort darf nur aus dem Startsymbol erzeugt werden."
	      ]

    else if not $ isEmptySet feps
    then Left $  unlines
	      [ "Regel(n) " ++ show feps
	      , "sind in monotoner Grammatik nicht gestattet."
	      , "Wenn aus dem Startsymbol ein leeres Wort abgeleitet wird,"
	      , "dann darf das Startsymbol in keiner rechten Regelseite vorkommen."
	      ]

    else if not $ null kurz 
    then Left $ unlines
	      [ "Regel(n) " ++ show kurz
	      , "sind in monotoner Grammatik nicht gestattet."
	      , "Die rechte Regelseite darf nicht krzer als die linke sein."
	      ]

    else Right $ "Das ist eine monotone Grammatik."

---------------------------------------------------------------------------

kontexte :: Grammatik 
	 -> (String, String) -> [ ((String, String), (Char, String)) ]
kontexte g (lhs, rhs) = do
    (pre, x : post) <- zerlegungen lhs
    guard $ x `elementOf` nichtterminale g
    let (vorn, rest) = splitAt (length pre) rhs
    guard $ pre == vorn
    let (mitte, hinten) = splitAt (length rest - length post) rest
    guard $ post == hinten
    return $ ((pre, post), (x, mitte))

kontextsensitiv :: Grammatik -> Either String String
kontextsensitiv g = let 
    zs = filterSet (null . kontexte g ) $ regeln g
 in if not $ isEmptySet zs
    then Left $  "Das ist keine kontextsensitive Grammatik."
	      ++ "Diese Regeln(n) haben nicht die Form  c V d -> c w d: "
	      ++ show zs
    else Right $ "Das ist eine kontextsensitive Grammatik."

---------------------------------------------------------------------------

kontextfrei :: Grammatik -> Either String String
kontextfrei g = let
    lang = filterSet ( (> 1) . length . fst ) $ regeln g
 in if not $ isEmptySet lang
    then Left $  "Das ist keine kontextfreie Grammatik."
	      ++ "Diese Regel(n) haben nicht die Form V -> w: " ++ show lang
    else Right $ "Das ist eine kontextfreie Grammatik."

-----------------------------------------------------------------------------

linear :: Grammatik -> Either String String
linear g = let
    schlecht = filterSet ( (>1) . length . filter (`elementOf` nichtterminale g) . snd ) 
	     $ regeln g
 in if not $ isEmptySet schlecht
    then Left $  "Das ist keine lineare Grammatik."
	      ++ "Diese Regel(n) enthalten rechts mehr als ein Nichtterminal: "
	      ++ show schlecht
    else Right $  "Das ist eine lineare Grammatik."

------------------------------------------------------------------------------

rechtslinear :: Grammatik -> Either String String
rechtslinear g = let
    schlecht = filterSet ( not . ( `elementOf` nichtterminale g ) . last . snd ) 
	     $ filterSet ( (> 0) . length . filter ( `elementOf` nichtterminale g ) . snd ) 
	     $ regeln g
 in if not $ isEmptySet schlecht
    then Left $  "Das ist keine rechtslineare Grammatik."
	      ++ "\nDiese Regel(n) sind nicht von der Form M -> A^* M " ++ show schlecht
    else Right $ "Das ist eine rechtslineare Grammatik."

------------------------------------------------------------------------------

linkslinear :: Grammatik -> Either String String
linkslinear g = let
    schlecht = filterSet ( not . ( `elementOf` nichtterminale g ) . head . snd ) 
	     $ filterSet ( (> 0) . length . filter ( `elementOf` nichtterminale g ) . snd ) 
	     $ regeln g
 in if not $ isEmptySet schlecht
    then Left $  "Das ist keine linkslineare Grammatik."
	      ++ "\nDiese Regel(n) sind nicht von der Form M -> M A^* " ++ show schlecht
    else Right $ "Das ist eine linkslineare Grammatik."

---------------------------------------------------------------------------

epsfrei :: Grammatik -> Either String String
epsfrei g = 
    let schlecht = filterSet ( \ (lhs, rhs) -> null rhs ) $ regeln g
    in	if isEmptySet schlecht
	then Right $ "Das ist eine Grammatik ohne Epsilon-Regeln."
	else Left  $ unlines
		   [ "Die Grammatik darf diese Epsilon-Regeln nicht enthalten:"
		   , show schlecht
		   ]

kettenfrei :: Grammatik -> Either String String
kettenfrei g = 
    let schlecht = filterSet ( \ (lhs, rhs) -> 
	 length rhs == 1 && head rhs `elementOf` nichtterminale g ) $ regeln g
    in	if isEmptySet schlecht
	then Right $ "Das ist eine Grammatik ohne Ketten-Regeln."
	else Left  $ unlines
		   [ "Die Grammatik darf diese Ketten-Regeln nicht enthalten:"
		   , show schlecht
		   ]

chomsky :: Grammatik -> Either String String
chomsky g = 
    let ok rhs = ( length rhs == 1 && head rhs `elementOf` terminale g )
              || ( length rhs == 2 && and [ x `elementOf` nichtterminale g 
					  | x <- rhs ] )
	schlecht = filterSet ( \ ( lhs, rhs) -> not (ok rhs) ) $ regeln g
    in	if isEmptySet schlecht
	then Right $ "Das ist eine Grammatik in Chomsky-Normalform"
	else Left  $ unlines
		   [ "Eine Grammatik in Chomsky-Normalform darf diese Regeln nicht enthalten:"
		   , show schlecht
		   ]

greibach :: Grammatik -> Either String String
greibach g = 
--    let ok rhs = ( length rhs == 0 || head rhs `elementOf` terminale g )
-- es darf auch keine Eps-Regel geben, ist nur fr eps \notin L(g) definiert
    let ok rhs = ( length rhs > 0 
		   && head rhs `elementOf` terminale g
		   && and [ x `elementOf` nichtterminale g | x <- tail rhs ]
		 )
	schlecht = filterSet ( \ ( lhs, rhs) -> not (ok rhs) ) $ regeln g
    in	if isEmptySet schlecht
	then Right $ "Das ist eine Grammatik in Greibach-Normalform"
	else Left  $ unlines
		   [ "Eine Grammatik in Greibach-Normalform darf diese Regeln nicht enthalten:"
		   , show schlecht
		   ]

---------------------------------------------------------------------------

final :: String -> Either String String -> Either String String
final msg (Left no) = Left $ no ++ "\nDas ist keine " ++ msg
final msg (Right yes) = Right $ yes ++ "\nDas ist eine " ++  msg

----------------------------------------------------------------------------

typ1 g = final "Typ-1-Grammatik"
--       $ und [ typ0 g, oder [ monoton g,  kontextsensitiv  g ] ] 
       $ und [ typ0 g, monoton g ] 
       
typ2 g = final "Typ-2-Grammatik"
       $ und [ typ0 g, kontextfrei g ]

typ3 g = final "Typ-3-Grammatik"
       $ und [ typ0 g, kontextfrei g, linear g
	     , oder [ linkslinear g, rechtslinear g ] ]

