module Graph where

import Set
import FiniteMap

-------------------------------------------------------------------------------

data Graph a  = Graph
	      { tafel     :: FiniteMap a (Knoten a)
	      , kanten    :: Set (Kante a)
	      }

data Knoten a = Knoten
              { idy       :: a
	      , info      :: String
              }

data Kante a  = Kante
	      { von       :: a
	      , nach      :: a
              } deriving (Eq, Ord)

-------------------------------------------------------------------------------

instance Eq  a => Eq  (Graph a)
               where x == y = (tafel x == tafel y) && (kanten x == kanten y)

instance Eq  a => Eq  (Knoten a) where x == y = idy x == idy y
instance Ord a => Ord (Knoten a) where x `compare` y = idy x `compare` idy y

-------------------------------------------------------------------------------

instance (Show a, Ord a) => Show (Graph a) where
    showsPrec p g = showString $ unlines 
		  $ [ "{ knoten = " ++ show (knotenl g)
		    , ", kanten = " ++ show (kanten g)
		    --, ", tafel  = " ++ show (fmToList  (tafel g))
		    , "}"
		    ]

instance (Show a, Ord a) => Show (Knoten a) where
    showsPrec p k = showString $ show (info k)
 
instance (Show a, Ord a) => Show (Kante a) where
    showsPrec p k = showString $ "(" ++ show (von k) ++ "," 
                                     ++ show (nach k) ++ ")"

-------------------------------------------------------------------------------
-- easy typing

kn :: a -> String -> (Knoten a)
kn a i = Knoten { idy = a, info = i }

ka :: a -> a -> (Kante a)
ka x y = Kante { von = x, nach = y }

knoten :: Ord a => Graph a -> Set a
knoten g = mkSet (knotenl g)

knotenl :: Ord a => Graph a -> [a]
knotenl g = keysFM (tafel g)
