module Graph_Util where

import Graph

import Set
import FiniteMap

import Monad ( guard )

-------------------------------------------------------------------------------

mapG :: (Ord a, Ord b) 
     => (String -> String) -> (a -> b) -> (Graph a -> Graph b)
mapG fi f g = Graph 
     { tafel = listToFM [ ( f i, v { idy = f (idy v), info = fi (info v) } )
			| (i, v) <- fmToList (tafel g)
			]
     , kanten = mapSet ( \ k -> ka (f (von k)) (f (nach k)) ) ( kanten g )
     }

abzhl :: Ord a => Graph a -> Graph Integer
abzhl g = 
    let fm  = listToFM $ zip ( knotenl g ) [ 1 .. ]
        f   = lookupWithDefaultFM fm (error "abzhl") 
    in  mapG id f g

gefhrliche_summe :: Ord a => Graph a -> Graph a -> Graph a
gefhrliche_summe g1 g2 = 
    Graph { tafel = plusFM (tafel g1) (tafel g2)
	  , kanten = kanten g1 `unionSet` kanten g2
	  }

summe :: (Ord a, Ord b) => Graph a -> Graph b -> Graph (Either a b)
summe g1 g2 =
    mapG ('L' :) Left g1 `gefhrliche_summe` mapG ('R' :) Right g2

-----------------------------------------------------------------------------

mapK :: Ord a
     => (Set (Kante a) -> Set (Kante a)) -> (Graph a -> Graph a)
mapK f g = Graph { tafel = tafel g, kanten = f ( kanten g ) }

verbinde :: Ord a => Graph a -> a -> a -> Graph a
verbinde g x y = mapK (`unionSet` mkSet [ ka x y ]) g

verbinde_symmetrisch :: Ord a => Graph a -> a -> a -> Graph a
verbinde_symmetrisch g x y = verbinde (verbinde g x y) y x

entferneKante :: Ord a => Graph a -> a -> a -> Graph a
entferneKante g x y = mapK (`minusSet` mkSet [ ka x y ]) g

entferneKante_symmetrisch :: Ord a => Graph a -> a -> a -> Graph a
entferneKante_symmetrisch g x y = entferneKante (entferneKante g x y) y x

rckwrts :: Ord a => Graph a -> Graph a
rckwrts = mapK (mapSet ( \ k -> ka (nach k) (von k) ))

symmetrisch :: Ord a => Graph a -> Graph a
symmetrisch g = mapK (`unionSet` kanten (rckwrts g)) g

reflexiv :: Ord a => Graph a -> Graph a
reflexiv g = mapK (`unionSet` mkSet [ ka x x | x<-(knotenl g) ]) g

unabhngig_auf :: Ord a => [ a ] -> Graph a
unabhngig_auf xs = 
    Graph { kanten = emptySet
	  , tafel  = listToFM [ (k, kn k "") | k <- xs ]
	  }

-- ohne Schlinge
komplement :: Ord a => Graph a -> Graph a
komplement g = 
   Graph { tafel  = tafel g 
	 , kanten = mkSet [ ka k k' 
                          | k <- (knotenl g) 
                          , k' <- (knotenl g)
                          , k < k' || k' < k
                          ] `minusSet` (kanten g)
	 }

clique_auf :: Ord a => [ a ] -> Graph a
clique_auf = komplement . unabhngig_auf

voll_auf :: Ord a => [ a ] -> Graph a
voll_auf = reflexiv . clique_auf

runde_auf :: Ord a => [ a ] -> [(a,a)]
runde_auf xs = zip xs (tail xs ++ [head xs])

kreis_auf :: Ord a => [ a ] -> Graph a
kreis_auf xs =
    foldr ( \ (x,y) g -> verbinde g x y) 
          (unabhngig_auf xs) (runde_auf xs)

istWeg :: Ord a => Graph a -> [ a ] -> Bool
istWeg g xs = isEmptySet (mkSet (map ( \ (x,y) -> ka x y ) (zip xs (tail xs)))
                         `minusSet` kanten g)

-------------------------------------------------------------------------------

vorgnger :: Eq a => (Graph a) -> a -> [ a ]
vorgnger g x = do
	  k @ Kante { } <- setToList $ kanten g
	  guard $ nach k == x
	  return $ von k

nachfolger :: Eq a => (Graph a) -> a -> [ a ]
nachfolger g x = do
	  k @ Kante { } <- setToList $ kanten g
	  guard $ von k == x
	  return $ nach k

ein_grad, aus_grad, grad :: Eq a => (Graph a) -> a -> Int
ein_grad g k = length $ vorgnger g k
aus_grad g k = length $ nachfolger g k
grad g k = ein_grad g k + aus_grad g k

neuerKnoten :: Ord a => Graph a -> a -> String -> Graph a
neuerKnoten g a i =
  Graph { tafel   = addToFM (tafel g) a (kn a i)
         , kanten = kanten g
         }

fehlendeKnoten :: Ord a => Graph a -> [a] -> Set a
fehlendeKnoten g xs = knoten g `minusSet` mkSet xs

mehrfacheKnoten :: Ord a => [a] -> Set a
mehrfacheKnoten xs = 
  mkSet [ x 
        | (x,y)<-(fmToList (addListToFM_C (+) emptyFM [ (x,1) | x<-xs ]))
        , y>1 ]

-------------------------------------------------------------------------------

entferneKnoten :: Ord a => Graph a -> a -> Graph a
entferneKnoten g k = teil g (setToList ((knoten g) `minusSet` mkSet [ k ]))

teil :: Ord a => Graph a -> [ a ] -> Graph a
teil g xs = let m = mkSet xs
                erlaubt x = x `elementOf` m
            in Graph { tafel  = filterFM ( \ x v -> erlaubt x ) (tafel g)
                     , kanten = filterSet ( \ k -> erlaubt (von k) && erlaubt (nach k)) (kanten g) 
                     }
