module Grundy where

import NFA
import FAU (smap)
import qualified Ops 
import Det
import Minimize

import qualified Bits (xor)
import Int

import Rewrite 
import Prepare

import Inter

import Set
import FiniteMap

import Monad ( guard )
import List ( inits )

type RS = [(String, String)]

-------------------------------------------------------------------

report :: RS -> MS -> IO ()
report rs ms0 = do
    putStrLn $ "for rewrite game " ++ show rs
    putStrLn $ "checking grundy subclasses:"
    visuals ms0
    let ms = prepare ms0
    putStrLn $ "\n\nprepared version:"
    visuals ms



    putStrLn $ "checking that prepared versions are disjoint"
    mapM_ ( \ (i, j) -> do
	putStrLn $ "\nfor (i,j) = " ++ show (i,j)
	putStrLn $ "M_i cap M_j is "
	print $ take 20 $ do
	    w <- take 1000 $ accepted (ms !! i)
	    guard $ is_accepted (ms !! j) w
	    return w

      ) [ (i,j) | i <- [1 .. length ms - 1], j <- [ 0 .. i-1] ]
    


    putStrLn $ "pre-images under rewriting are:"
    let prems = map ( rewrites ( inverse rs ))  ms
    visuals prems

    putStrLn $ unlines
	     [ "checking that each w in M_i has an option"
	     , "in each smaller class M_j"
	     ]
    mapM_ ( \ (i, j) -> do
	putStrLn $ "\nfor (i,j) = " ++ show (i,j)
	putStrLn $ "M0_i setminus R^-(M_j) is "
	-- das dauert leider zu lange:
	-- print $ minus_det (ms !! i) (prems !! j)
	print $ take 20 $ do
	    w <- take 100 $ accepted (ms0 !! i)
	    guard $ not $ is_accepted (prems !! j) w
	    return w

      ) [ (i,j) | i <- [1 .. length ms - 1], j <- [ 0 .. i-1] ]

    putStrLn $ "post-images (of ms0) under rewriting are:"
    let poms = map ( rewrites rs )  ms0
    visuals poms

    putStrLn $ unlines
	     [ "checking that each Mi0 has all options"
	     , "in M0 .. M{i-1} union R^-(Mi)"
	     ]

    let comps = do
	    xs <- inits ms
	    let k = length xs
	    return $ foldr1 ( \ x y -> normalize $ Ops.union x y )
		    (prems !! k : xs)

    mapM_ ( \ (o,c) -> do
	  putStrLn "\n****"
	  print $ do
		w <- take 100 $ accepted o
		guard $ not $ is_accepted c w
		return w
	) $ zip poms comps


-------------------------------------------------------------------

inverse :: RS -> RS
inverse rs = do (l,r) <- rs; return (r,l)

check_first :: RS -> MS -> MS
-- berechnet liste von  M_i \setminus ( R^-(M_0) \cap .. \cap R^-(M_{i-1}) )
-- sollte alles leer sein
check_first rs ms = do
    let rs' = inverse rs
	ms' = map ( normalize . det . rewrites rs' ) ms
	ds = scanl1 (\ x y -> normalize $ Ops.intersection x y) ms'
    ( m, d ) <- zip (tail ms) ds
    return $ minus m d -- d sollte bereits det. sein




