module Prepare where

import NFA
import FAU (smap)
import qualified Ops 
import Det
import Minimize

import qualified Bits (xor)
import Int

import Rewrite 

import Inter

import Set
import FiniteMap

xor :: Int -> Int -> Int
xor x y = int32ToInt $ Bits.xor (intToInt32 x) (intToInt32 y)

type MS = [ NFA Int ]

visual :: NFA Int -> IO ()
visual m = do
    print m
    print $ take 20 $ accepted m

visuals :: MS -> IO ()
visuals = ( mapM_ $ \ (g,m) -> do
    putStrLn $ "\nGrundy value " ++ show g
    visual m
  ) . zip [0..]

------------------------------------------------------------------------

cyclic :: MS -> MS
-- einput: vier sprachen (grundy-teilklassen) M0, M1, M2, M3 \subset {0,1}^*
-- ausput: sprachen Mi' \subset {0,1,2}^*
-- mit 2 als disjunktem summen-symbol
cyclic ms | length ms == 4 = do
    let range = [ 0 .. 3 ] -- diese werden modifiziert (verlngert)
	top = 4 -- diese kopie wird nicht gendert, 
		-- dort sind auch die endzustnde

    -- automaten vervielfachen
    let mms = do (i,m) <- zip range ms
		 j <- top : range
		 return $ amap ((,) (i,j)) m

    let starters k = do
	    i <- range
	    q <- setToList $ starts (ms !! i)
	    return ((i,k),q) 
	  ++ do q <- setToList $ starts (ms !! k); return ((k,top),q)

    -- querverbindungen entspr. disjunkter summe von grundy-werten
    let ts  = collect $ do 
	         i <- range ; j <- range
		 p <- setToList $ finals (ms !! i)
		 q <- starters $ xor i j
		 return ( ((i,j),p), '2', q )

    i <- range
    return $ minimize 
	   $ normalize
	   $ NFA { states = unionManySets $ map states mms 
		 , starts = mkSet $ starters i
		 , finals = unionManySets $ do
			  j <- range
			  return $ smap ( (,) (j,top) )
				 $ finals ( ms !! j )
		 , trans = foldr1 ( plusFM_C union )
			 $ ts : map trans mms
		 }


loops :: NFA Int -> NFA Int
loops a = 
    let ms = [ amap ( \ q -> (q,i) ) a | i <- [0,1] ]
	ls = collect $ do 
	        q <- setToList $ states a
		[ ((q,0), '0', (q,1)),  ((q,1), '0', (q,0)) ]
    in	  minimize 
	$ normalize
	$ NFA { states = unionManySets $ map states ms
	    , starts = starts $ ms !! 0
	    , finals = finals $ ms !! 0
	    , trans = plusFM_C union ls $ trans $ ms !! 0 
	    } 

suffice :: NFA Int -> NFA Int
-- addiere fr grundy-wert unschdlichen suffix
suffice a = 
    let suf = inter std . read $ "0^* (1^2 0^*)^*"
    in	Ops.dot a suf

couple :: [NFA Int] -> [NFA Int]
-- man kommt vom einem zum anderen durch Vorschalten einer "1"
couple [ a0, b0 ] = do
    let ab = do x <- [ a0, b0 ]
		return $ Ops.punkt (Ops.neu x) x
	range = [ 0 .. 1 ]
	ms = do (i,x) <- zip range ab
		return $ amap ((,) i) x
	ts = collect $ do 
		p <- setToList $ starts $ ms !! 0
		q <- setToList $ starts $ ms !! 1
	        [ (p, '1', q), (q, '1', p) ]
    i <- range
    return $ normalize
	   $ NFA { states = unionManySets $ map states ms
		 , starts = starts $ ms !! i
		 , finals = unionManySets $ map finals ms
		 , trans  = foldr1 (plusFM_C union)
			  $ ts : map trans ms
		 }

prepare :: MS -> MS
prepare ms = 
    let [a,b,c,d] = map suffice ms
	[p,q] = couple [a,b]
	[r,s] = couple [c,d]
	cs = cyclic [p,q,r,s]
    in	map loops cs
    







