module NPDA 

-- nicht-deterministischer Kellerautomat

( module NPDA
, module Set
, module FiniteMap
)

where

-- $Log: NPDA.hs,v $
-- Revision 1.1.1.1  2002/05/24 10:46:48  challenger
-- start
--
-- Revision 1.2  2002/01/13 23:03:33  autotool
-- kellerautomaten: nutze toDoc, akzeptanzmodus sichtbar
--

import Set
import FiniteMap
import Schichten
import Size

import ToDoc
import ReadSet
import ReadFM

data Modus z = Leerer_Keller | Zustand (Set z) 
     deriving (Show, Read)

instance ToDoc [z] => ToDoc (Modus z) where
     toDoc Leerer_Keller = text "Leerer_Keller"
     toDoc ( Zustand zs ) = text "Zustand" <+> parens (toDoc zs)


data NPDA x y z = 
     NPDA { eingabealphabet  :: Set x 
	  , kelleralphabet   :: Set y 
	  , zustandsmenge    :: Set z 
	  , tafel	     :: FiniteMap (Maybe x, z, y) (Set (z, [y]))
	  , startzustand     :: z
	  , startsymbol	     :: y
	  , akzeptiert	     :: Modus z
	  }
     deriving (Read, Show)

instance (ToDoc (Maybe x), ToDoc x, ToDoc y, ToDoc z,
	 ToDoc [x], ToDoc [y], ToDoc [z]) 
	 => ToDoc (NPDA x y z) where
    toDoc a = text "NPDA" <+> braces ( fsep $ punctuate comma
       [ text "eingabealphabet" <+> equals <+> toDoc ( eingabealphabet a )
       , text "kelleralphabet" <+> equals <+> toDoc ( kelleralphabet a )
       , text "zustandsmenge" <+> equals <+> toDoc ( zustandsmenge a )
       , text "startsymbol" <+> equals <+> toDoc ( startsymbol a )
       , text "startzustand" <+> equals <+> toDoc ( startzustand a )
       , text "akzeptiert" <+> equals <+> toDoc ( akzeptiert a )
       , text "tafel" <+> equals <+> toDoc ( tafel a )
       ] )

instance Size (NPDA x y z) where
    size a = sum $ do
	 ( x, ys ) <- fmToList $ tafel a
	 y <- setToList ys
	 return 1


lookupset :: Ord a => FiniteMap a (Set b) -> a -> Set b
lookupset fm x = case lookupFM fm x of
    Just m -> m; Nothing -> emptySet


check :: (Ord x, Ord y, Ord z, Show x, Show y, Show z)
      => NPDA x y z -> Either String String
check a = case 

      [ "das Eingabe-Alphabet ist leer" | isEmptySet (eingabealphabet a) ]
      ++ 

      [ "Fehler in Regel " ++ show r ++ ": " ++ msg
      | r @ ((mx,z,y),zys) <- fmToList $ tafel a
      , (z', y') <- setToList zys
      , msg <- [ show x ++ " gehrt nicht zum Eingabealphabet" 
	       | x <- case mx of Just x -> [x]; Nothing -> []
	       , not ( x `elementOf` eingabealphabet a )
	       ] 
	       ++
	       [ show t ++ " gehrt nicht zum Kelleralphabet" 
	       | t <-  y : y'
	       , not ( t `elementOf` kelleralphabet a )
	       ] 
	       ++ 
	       [ show t ++ " gehrt nicht zur Zustandsmenge" 
	       | t <- [ z, z' ]
	       , not ( t `elementOf` zustandsmenge a )
	       ]
      ]

      ++ 

      [ "das Startsymbol " ++ show s ++ " gehrt nicht zum Kelleralphabet"
      | let s = startsymbol a
      , not (s `elementOf` kelleralphabet a)
      ]
      ++ 
      [ "der Startzustand " ++ show z ++ " gehrt nicht zur Zustandsmenge"
      | let z = startzustand a
      , not (z `elementOf` zustandsmenge a)
      ]
      ++ 
      [ "der Endzustand " ++ show z ++ " gehrt nicht zur Zustandsmenge"
      | Zustand zs <- [ akzeptiert a ]
      , z <- setToList zs
      , not (z `elementOf` zustandsmenge a)
      ]

  of  []  -> Right "Das ist wirklich ein nichtdeterministischer Kellerautomat."
      msg -> Left $ unlines 
	  $ [ "Das ist kein nichtdeterministischer Kellerautomat." ]
	    ++ msg









