module Turing

-- nicht-deterministische Turingmaschine

( module Turing
, module Set
, module FiniteMap
)

where

-- $Log: Turing.hs,v $
-- Revision 1.1.1.1  2002/05/24 10:46:48  challenger
-- start
--
-- Revision 1.3  2002/04/11 09:25:50  autotool
-- size, fun
--
-- Revision 1.2  2002/04/08 11:32:25  autotool
-- turing updates
--

import Set
import ToDoc
import FiniteMap
import Schichten
import Monad (guard)

import Size

data Bewegung = L | O | R
     deriving (Eq, Show, Ord)

instance ToDoc Bewegung where
    toDoc = text . show

data Turing y z = 
     Turing { eingabealphabet  :: Set y
	    , arbeitsalphabet  :: Set y
	    , leerzeichen	     :: y
	    , zustandsmenge    :: Set z 
	    , tafel	     :: FiniteMap (y, z) (Set (y, z, Bewegung))
	    , startzustand     :: z
	    , endzustandsmenge :: Set z
	    }

class (Ord y, Ord z, Show y, Show z, ToDoc y, ToDoc z
	 , ToDoc [y], ToDoc [z]) => TUM y z 
    -- ohne methoden, soll nur die constraints aufsammeln
instance TUM Char Integer


instance Size (Turing y z) where
    size = size . tafel


instance TUM y z
	 => ToDoc (Turing y z) where
   toDoc m = text "Turing" <+> braces ( fsep $ punctuate comma
       [ text "eingabealphabet" <+> equals <+> toDoc (eingabealphabet m)
       , text "arbeitsalphabet" <+> equals <+> toDoc (arbeitsalphabet m)
       , text "leerzeichen" <+> equals <+> toDoc (leerzeichen m)
       , text "zustandsmenge" <+> equals <+> toDoc (zustandsmenge m)
       , text "startzustand" <+> equals <+> toDoc (startzustand m)
       , text "endzustandsmenge" <+> equals <+> toDoc (endzustandsmenge m)
       , text "tafel" <+> equals <+> toDoc (tafel m)
       ] )

instance TUM y z
	 => Show (Turing y z) where
    show = render . toDoc


lookupset :: Ord a => FiniteMap a (Set b) -> a -> Set b
lookupset fm x = case lookupFM fm x of
    Just m -> m; Nothing -> emptySet


check :: TUM y z
      => Turing y z -> Either String String
check m = case 

      [ "das Eingabe-Alphabet ist leer" 
      | isEmptySet (eingabealphabet m) 
      ]
      ++ 
      [ "das Leerzeichen gehrt zum Eingabealphabet "	
      | leerzeichen m `elementOf` eingabealphabet m
      ]
      ++ 
      [ "das Leerzeichen gehrt nicht zum Arbeitsalphabet "	
      | not ( leerzeichen m `elementOf` arbeitsalphabet m)
      ]
      ++
      [ "das Eingabealphabet " ++ show e 
	++ " ist keine Teilmenge des Arbeitsalphabets " ++ show a
      | let e = eingabealphabet m
      , let a = arbeitsalphabet m
      , not (isEmptySet (e `minusSet` a))
      ]
      ++
      [ "Fehler in der Programmregel " ++ show r ++ ": " ++ msg
      | r @ ((y,z),yzbs) <- fmToList $ tafel m
      , (y',z',b) <- setToList yzbs
      , msg <- [ show t ++ " gehrt nicht zum Arbeitsalphabet" 
	       | t <- [ y, y' ] 
	       , not ( t `elementOf` arbeitsalphabet m )
	       ] 
	       ++ 
	       [ show t ++ " gehrt nicht zur Zustandsmenge" 
	       | t <- [ z, z' ]
	       , not ( t `elementOf` zustandsmenge m )
	       ]
      ]
      ++ 
      [ "der Startzustand " ++ show z ++ " gehrt nicht zur Zustandsmenge"
      | let z = startzustand m
      , not (z `elementOf` zustandsmenge m)
      ]
      ++ 
      [ "der Endzustand " ++ show z ++ " gehrt nicht zur Zustandsmenge"
      | z <- setToList $ endzustandsmenge m
      , not (z `elementOf` zustandsmenge m)
      ]

  of  []  -> Right "Das ist wirklich eine nichtdeterministische Turingmaschine."
      msg -> Left $ unlines 
	  $ [ "Das ist keine nichtdeterministische Turingmaschine." ]
	    ++ msg

deterministisch :: TUM y z
      => Turing y z -> Either String String
deterministisch m = 
    let mehr = do
	    r @ ( yz, s ) <- fmToList $ tafel m
	    guard $ cardinality s > 1
	    return r
    in	case mehr of
	    [] -> Right "diese Maschinen ist deterministisch"
	    rs -> Left  $ unlines 
			( "diese Regeln sind nicht deterministisch:"
			: map show rs )

