module Alternate.Check where

import Alternate.Maschine
import Alternate.Wahrheit
import Alternate.Status

import qualified Size

import Baum
import Language
import Language.Object

import ToDoc
import Set
import List (partition)

import Right -- old-style autotool
import Wrong

-- generisch, also kein import Bild!

check_language :: ( Maschine m o k,  Size.Size m, Edit o
		  , ToDoc o, ToDoc [a]
		  , Object a o )
	       => Int -> Int -> Int
	       -> Language a o -> m
	       -> IO String
check_language size number depth lang mach = do
    let s = Size.size mach
    let msg = "OK # Size: " ++ show s

    os <- samples lang number size
    ps <- mapM edits os
    rs <- samples (complement lang) number size

    let reports = map ( check_object depth lang mach ) $ os ++ ps ++ rs
    let eins (a,b,c) = a; zwei (a,b,c) = b; drei (a,b,c) = c
    let yeah = filter ( (== Ja) . zwei ) reports
    let noh  = filter ( (== Nein) . zwei ) reports
    let may  = filter ( (== Vielleicht) . zwei ) reports
    putStrLn $ unlines
	     [ "Guten Tag. Gesucht war ein Automat fr"]
    port lang
    putStrLn $ unlines
	     [ "\nSie haben eingesandt:"
	     , render $ toDoc mach
	     , "\nich generiere " ++ show number ++ " Objekte"
	     ++ " ab der Gre " ++ show size
	     , "und simuliere Rechnungen von bis zu " ++ show depth ++ " Schritten."
	     
	     , "\n(Ich zeige im folgenden nur die jeweils 5 ersten Objekte)"
	     , "\nIhr Automat behandelt korrekt:"
	     , render $ toDoc $ take 5 $ map eins yeah
	     , "\nIhr Automat behandelt inkorrekt:"
	     , render $ toDoc $ take 5 $ map eins noh
	     , "\nIhr Automat rechnet zu lange fr:"
	     , render $ toDoc $ take 5 $ map eins may
	     ]
    putStrLn $ unlines
	     [ "Hier sehen Sie einige (gekrzte) Berechnungsbume."
	     , ""
	     , "Der Simulator schneidet dabei stillschweigend Teilbume ab,"
	     , "sobald sich auf einem Suchpfad eine Konfiguration wiederholt"
	     , "(siehe source code in Alternate/Check.hs)"
	     , ""
	     ]
    let focus = take 2 yeah ++ take 2 noh ++ take 2 may
    mapM_ (putStrLn . render . inform) focus

    if null noh 
       then right_with msg 
       else wrong


inform (a, b, c) = 
       vcat [ text "Eingabe" <+> equals <+> toDoc a
	    , text "Baum" <+> equals <+> display c
	    ]

---------------------------------------------------------------------------

check_object :: Maschine m o k
	     => Int -> Language a o -> m -> o 
	     -> (o, Wahrheit,  Baum (Maybe k, Wahrheit))
check_object depth lang mach b =
    let v = contains lang b
	( w, t ) = check depth b mach
	r = case w of
		 Vielleicht -> w
		 _ -> if bool w == v then Ja else Nein
    in	( b, r, t )
		

check :: Maschine m o k 
      =>       Int -> o -> m -> ( Wahrheit,  Baum (Maybe k, Wahrheit) )
-- testet, ob die Maschine m das Objekt o akzeptiert.
-- benutzt suche ber konfigurationsbaum der tiefe <= d
check d o m =
    let b = clip d $ baum m o 
	e = clipped_erfolgreich emptySet b
    in	( snd (wurzel e), e )

-- trick: knoten auf aktuellem pfad aufheben (in schon)
-- und abschneiden, falls wir einen zum zweiten mal treffen.
clipped_erfolgreich 
    :: (Hat_Status k , Ord k )
    => Set k -> Baum (Maybe k) -> Baum (Maybe k, Wahrheit)
clipped_erfolgreich schon b =
    case wurzel b of
	 Nothing -> Baum { wurzel = (Nothing, Vielleicht), kinder = [] }
	 Just w	 ->
	    if w `elementOf` schon
	    then Baum { wurzel = ( Nothing, Nein ), kinder = [] }
	    else
	         let
		     jetzt = addToSet schon w
		     bs = map (clipped_erfolgreich jetzt) $ kinder b
		     es = map (snd . wurzel) bs
		     op = case status w of
			  Existenziell -> findmax; Universell -> findmin
		 in  Baum { wurzel = (wurzel b, op es) , kinder = bs }





	
    
