module Baum.AndOr where

import Baum
import Language

import ToDoc
import Set
import Random
import Some
import Monad (guard)

t :: Language Char (Baum Char)
-- alle Formelbume mit wahrheitswert 1
t = Language
    { name = "T"
    , definition = braces 
		 $ text 
		 $ "t : t in Term(0,1,and,or), wert(t)=1"
    , alphabet = mkSet "01ao"
    , contains = \ t ->  wert t
    , sample      = \ c n -> sequence $ replicate c $ some_t True  n
    , anti_sample = \ c n -> sequence $ replicate c $ some_t False n
    }      
	     
wert :: Baum Char -> Bool
wert t = 
    let vs = map wert $ kinder t
    in case wurzel t of
     '0' -> False ; '1' -> True
     'a' -> and vs; 'o' -> or vs
	       
some_t :: Bool -> Int -> IO (Baum Char)
-- irgendein baum mit wert w, gre n
some_t w n =
    if n < 2
    then return $ Baum { wurzel = if w then '1' else '0'
		       , kinder = [] 
		       }
    else do
	 let trips = do
		   ( s, f ) <- [ ('a', (&&)), ('o', (||)) ]
		   x <- [ False .. True ] ; y <- [ False .. True ]
		   guard $ f x y == w
		   return ( s, x, y )
	 k <- randomRIO (0, length trips - 1)
	 let ( s, x, y ) = trips !! k
	 par <- somepartition   2 n
	 ts <- sequence $ do	      
		(v, s) <- zip [x,y] par
		return $ some_t v s
	 return $ Baum { wurzel = s
		       , kinder = ts
		       }

