module Baum.Engelfriet where

import Baum
import Language

import ToDoc
import Set
import Random


l :: Language Char (Baum Char)
l = Language
    { name = "L"
    , definition = braces $ text "t : rand (t) in a^* b a^*"
    , alphabet = mkSet "ab@"
    , contains = \ t ->  
	       let xs = rand t; n = length xs
		   count c = length $ filter (== c) xs
	       in  count 'a' == n - 1 && count 'b' == 1
    , sample = \ c n -> sequence $ replicate c $ some_l n
    , anti_sample = anti l
    }      
	     
some_l :: Int -> IO (Baum Char)
some_l n =
    if n < 2 then return $ Baum { wurzel = 'b', kinder = [] }
    else do
	 x <- randomRIO (1 , n - 2) ; let y = n - 1 - x
	 xx : _ <- sample l 1 x ; yy : _ <- sample a 1 y
	 flag <- randomRIO (False, True)
	 return $ Baum { wurzel = '@'
		       , kinder = if flag then [xx,yy] else [yy,xx] 
		       }

----------------------------------------------------------------------

r :: Language Char (Baum Char)
r = Language
    { name = "R"
    , definition = braces $ text "R -> a | b | @(L , R)"
    , alphabet = mkSet "ab@"
    , contains = \ t ->  case kinder t of
	  [] -> wurzel t `elem` "ab"
	  [x, y] -> contains l x && contains r y
	  _ -> False
    , sample = \ c n -> sequence $ replicate c $ some_r n
    , anti_sample = anti r
    }      
	     
some_r :: Int -> IO (Baum Char)
some_r n =
    if n < 2 then do
       k <- randomRIO (0,1); let x = "ab" !! k
       return $ Baum { wurzel = x, kinder = [] }
    else do
	 x <- randomRIO (1 , n - 2) ; let y = n - 1 - x
	 xx : _ <- sample l 1 x ; yy : _ <- sample r 1 y
	 return $ Baum { wurzel = '@'
		       , kinder = [xx,yy]
		       }

----------------------------------------------------------------------

a :: Language Char (Baum Char)
a = Language
    { name = "A"
    , definition = braces $ text "t : rand (t) in a^*"
    , alphabet = mkSet "ab@"
    , contains = \ t ->  
	       let xs = rand t; n = length xs
		   count c = length $ filter (== c) xs
	       in  count 'a' == n
    , sample = \ c n -> sequence $ replicate c $ some_a n
    , anti_sample = sample ab
    }
	     
some_a :: Int -> IO (Baum Char)
some_a n =
    if n < 2 then return $ Baum { wurzel = 'a', kinder = [] }
    else do
	 x <- randomRIO (1 , n - 2) ; let y = n - 1 - x
	 xx : _ <- sample a 1 x ; yy : _ <- sample a 1 y
	 return $ Baum { wurzel = '@'
		       , kinder = [xx,yy]
		       }

----------------------------------------------------------------------

ab :: Language Char (Baum Char)
ab = Language
    { name = "AB"
    , definition = braces $ text "t : rand (t) in (a+b)^*"
    , alphabet = mkSet "ab@"
    , contains = \ t ->  
	       let xs = rand t; n = length xs
		   count c = length $ filter (== c) xs
	       in  count '@' == 0
    , sample = \ c n -> sequence $ replicate c $ some_ab n
    , anti_sample = anti ab
    }
	     
some_ab :: Int -> IO (Baum Char)
some_ab n =
    if n < 2 then  do
       k <- randomRIO (0,1); let x = "ab" !! k
       return $ Baum { wurzel = x, kinder = [] }
    else do
	 x <- randomRIO (1 , n - 2) ; let y = n - 1 - x
	 xx : _ <- sample ab 1 x ; yy : _ <- sample ab 1 y
	 return $ Baum { wurzel = '@'
		       , kinder = [xx,yy]
		       }

---------------------------------------------------------------------------
