-- $Id: Lay.hs,v 1.2 2002/07/03 14:21:16 joe Exp $


module Baum.Lay 

( Baum (..)
)

where

import Baum.Type
import qualified ToDoc
import Size

instance Size Char where size c = 1

-------------------------------------------------------------------------

type Item = String

type Line = [ Item ]

data Lay = Lay { contents :: [ Line ]
	       , width    :: Int 
	       , height   :: Int
	       }

instance Size Lay where size l = height l * width l

--------------------------------------------------------------------------

instance ToDoc.ToDoc Lay where
    toDoc l = 
	ToDoc.vcat $ map (ToDoc.hcat . map ToDoc.text) $ contents l

--------------------------------------------------------------------------

mkBin :: Baum Item -> Baum Item
mkBin top @ Baum { wurzel = k, kinder = cs } 
    | length cs <= 2  =  top { kinder = map mkBin cs }
mkBin top @ Baum { wurzel = k, kinder = x : ys } =
    top { kinder = [ x, mkBin $ Baum { wurzel = "+", kinder = ys } ] }

lay :: Baum Item -> Lay
lay top @ Baum { wurzel = k, kinder = xs } = 
    case xs of
          [ ] -> item k
	  [x] -> item k `atop` lay  x
	  [x , y] -> arrange k (lay x) (lay y)
	  _ -> lay ( mkBin top )

instance Show a => ToDoc.ToDoc (Baum a) where
    toDoc = ToDoc.toDoc . lay . fmap show

instance Show a => Show (Baum a) where
    show = ToDoc.render . ToDoc.toDoc

--------------------------------------------------------------------------

-- horizontal or vertical
arrange :: Item -> Lay -> Lay -> Lay
arrange k x y = 
    let v = ( k `left_hook` y ) `atop` x
	h = ( k `top_hook` x ) `aside` y
    in	if size v <= size h then v else h

par :: Line -> Lay
par its = Lay { contents = [ its ]
	      , width = sum (map size its)
	      , height = 1
	      }

pack :: [ Line ] -> Lay
pack lines = Lay { contents = lines
		 , width = maximum $ 0 : [ size line | line <- lines ]
		 , height = length lines
		 }

item :: Item -> Lay
item x = par [x]


-- vertical composition ---------------------------------------------------

-- separated by blank line
vsep = foldr1 ( vsepBy $ "" )

-- linked
atop = vsepBy $  "|"

vsepBy :: Item -> Lay -> Lay -> Lay
vsepBy it o u = vcat o $ vcat (item it) u

-- not separated
vcat :: Lay -> Lay -> Lay
vcat o u = Lay 
	 { contents = contents o ++ contents u
	 , width = max (width o) (width u)
	 , height = height o + height u
	 }

top_hook :: Item -> Lay -> Lay
-- result is this:
--    item --------
--    |
--    here is the
--    contents of l
top_hook i l = 
    let hunk = [ i, "-" ]
	fill = 0 `max` ( width l - size hunk )
	tab =  [ replicate fill '-' ]
    in	par ( hunk ++ tab ) `atop` l
 

-- horizontal composition ------------------------------------------------


hcat :: Lay -> Lay -> Lay
hcat l r = 
   let (pre, post) = splitAt (height r) (contents l)
       w = maximum [ size line | line <- pre ]
       extend line = line ++ [ replicate (w - size line) ' ' ]
       fill xs = xs ++ repeat [ "" ]
       joined = zipWith (++) (map extend $ fill pre) (contents r)
   in  pack $ joined ++ post 

hsepBy :: Item -> Lay -> Lay -> Lay
hsepBy it l r = hcat l $ hcat (item it) r

hsep :: Lay -> Lay -> Lay
hsep = hsepBy ( " " )

aside :: Lay -> Lay -> Lay
aside = hsepBy ( "--" )
	
left_hook :: Item -> Lay -> Lay
-- result is this:
--    item --- here
--    |        is the contents
--    |        of l
left_hook i l = 
    let hunk = [ i,  "---" ]
	tab =  [  '|' : take (size hunk - 1) (repeat ' ') ]
	x : ys = contents l
    in  Lay  { contents = ( hunk ++ x )
			: [ tab ++ y | y <- ys ]
	     , width = size hunk + width l
	     , height = height l
	     }






