module Baum.Majority where

import Baum
import Language

import ToDoc
import Set
import Random
import Some


m :: Int -> Language Char (Baum Char)
-- mehrheitssprache mit aritt k
m k = Language
    { name = "M_" ++ show k
    , definition = braces 
		 $ text 
		 $ "t : t in Term(0,1,m^" ++ show k ++ "), wert(t)=1"
    , alphabet = mkSet "01m"
    , contains = \ t ->  wert t
    , sample      = \ c n -> sequence $ replicate c $ some_m True k n
    , anti_sample = \ c n -> sequence $ replicate c $ some_m False k n
    }      
	     
wert :: Baum Char -> Bool
wert t = case kinder t of
     [] -> case wurzel t of
		'0' -> False ; '1' -> True
     ts -> let ws = map wert ts
	       os = filter id ws
	   in  2 * length os >= length ws
	       
some_m :: Bool -> Int -> Int -> IO (Baum Char)
-- irgendein baum mit wert w, aritt ar, gre n
some_m w arity n =
    if n < arity
    then return $ Baum { wurzel = if w then '1' else '0'
		       , kinder = [] 
		       }
    else do
	 per <- somepermutation arity
	 par <- somepartition   arity n
	 let water = (arity + 1) `div` 2
	 bnd <- randomRIO $ if w then (water, arity) else (0, water - 1)
	 -- alle mit <= bnd werden wahr gesetzt
	 ts <- sequence $ do	      
		( e, a ) <- zip per par
		return $ some_m ( e <= bnd ) arity a
	 return $ Baum { wurzel = 'm'
		       , kinder = ts
		       }

