module Baum.Type where

import Language.Object
import Language.Address
import Language.Edit

import ToDoc
import Set
import Maybe
import Monad (guard)
import Random

import Size

data Baum a = Baum { wurzel :: a
		     , kinder :: [ Baum a ]
		     }

-- lieber den layouter benutzen:

-- instance ToDoc a => ToDoc (Baum a) where
--     toDoc b = text "Baum" 
-- 	    <+> braces ( fsep $ punctuate comma 
-- 	    [ text "wurzel" <+> equals <+> toDoc ( wurzel b )
-- 	    , text "kinder" <+> equals <+> toDoc ( kinder b )
-- 	    ] )

-- instance ToDoc a => Show (Baum a) where
--     show = render . toDoc


instance Functor Baum where
    fmap f t = Baum { wurzel = f $ wurzel t
		    , kinder = map (fmap f) $ kinder t
		    }

subtrees :: Baum a -> [ Baum a ]
subtrees t = t : ( concat $ map subtrees $ kinder t )

innen :: Baum a -> [ Baum a ]
innen = filter (not . null . kinder) . subtrees 

rand :: Baum a -> [ a ]
rand = map wurzel . filter (null . kinder) . subtrees 

keys :: Baum a -> [ a ]
keys = map wurzel . subtrees

instance Size a => Size (Baum a) where
    size = sum . keys . fmap size

--------------------------------------------------------------------------

type Position = [ Int ]

data Direction = Up | Down Int
     deriving (Eq, Ord, Show, Read)

instance ToDoc Direction where 
     toDoc = text . show

instance Directed Position Direction where
    move p Up = init p
    move p (Down k) = p ++ [k]

instance Ord a => Object a (Baum a) where
    someIO s alpha = do
        let xs = setToList alpha
	k <- randomRIO (0, length xs - 1)
	let x = xs !! k
	if s > 2
	   then do sl <- randomRIO (1, s - 1)
		   let sr = s - 1 - sl
		   l <- someIO sl alpha
		   r <- someIO sr alpha
		   return $ Baum { wurzel = x, kinder = [ l, r ] }
	   else return $ Baum { wurzel = x, kinder = [] }

instance Address a (Baum a) Position where
    peek b [] = Just $ wurzel b
    peek b (k : ks) = do
        guard $ 0 <= k && k < length ( kinder b )
	peek ( kinder b !! k ) ks

instance Top (Baum a) Position where
    top b = []

instance Whoami Position where
    whoami p = case p of
	   [] -> ""
	   xs -> show (last xs)

-------------------------------------------------------------------------

positions :: Baum a -> [ Position ]
positions t = [] : do
    (k, s) <- zip [0 ..] $ kinder t
    p <- positions s
    return (k : p)

poke :: Baum a -> Position -> Baum a -> Baum a
poke t [] s = s
poke t (k : ks) s =
    let (pre, this: post) = splitAt k (kinder t)
    in	t { kinder = pre ++ poke this ks s : post }

pick :: Baum a -> Position -> Baum a
pick t [] = t
pick t (k : ks) =
    let (pre, this: post) = splitAt k (kinder t)
    in	pick this ks

instance Edit (Baum a) where
    edit t = do
	 let ps = positions t
	 k <- randomRIO (0, length ps - 1)
	 i <- randomRIO (0, length ps - 1)
	 return $ poke t (ps !! k) $ pick t (ps !! i)

-- zum Testen (der Ausgabe) -----------------------------------------------

fib :: Int -> Baum Int
fib n
    | n < 2 = Baum { wurzel = n, kinder = [] }
    | otherwise = Baum { wurzel = n, kinder = [ fib (n-1), fib (n-2) ] }
