module Bild.Center where

import Bild
import Language

import Random
import Set
import ToDoc

center0 :: Language Char (Bild Char)
center0 = Language 
     { name = "CENTER0"
     , definition = text "{ b : b in ODD-SQUARES(0,1) und ( b_xy = 1 ) <=> (x = y = Mitte) }"
     , alphabet     = mkSet "01"
     , contains = \ b -> let m = breit b `div` 2 in
	    breit b ==  2*m+1 && hoch b == 2*m+1  && and ( do
		  x <- [ 0 .. breit b - 1 ]
		  y <- [ 0 .. hoch  b - 1 ]
		  return $ b `at` (x,y) == if x == m && y == m 
					   then '1' else '0' )
     , sample = \ c n -> return $ do
           m <- [ n `div` 2 .. n `div` 2 + c - 1]
	   let b = uniform '0' (2*m+1, 2*m+1)
	   return $ tos b [ ((m,m), '1') ]
     , anti_sample = \ c n -> do
	  let g = n `div` 2
          -- kein quadrat
          let noquad = uniform '0' (n, n+1) `to` ((g, g), '1')
          -- kein ungerades quadrat
	  let noung =  uniform '0' (2*g,2*g) `to` ((g, g), '1')
	  -- einige 1, aber irgendwo
	  cands  <- sequence $ replicate c $ do
	      let u = 2 * g + 1
	      n <- randomRIO (0, 4)
	      xys <- sequence $ replicate n 
		   $ sequence $ replicate 2 $randomRIO (0, u-1)
	      return $ tos (uniform '0' (u,u)) [ ((x,y), '1') | [x,y] <- xys ]
	  return $ filter (not . contains center0) $ noquad : noung : cands
     }


center :: Language Char (Bild Char)
center = Language 
     { name = "CENTER"
     , definition = text "{ b : b in ODD-SQUARES(0,1) und (x = y = Mitte) => ( b_xy = 1 ) }"
     , alphabet     = mkSet "01"
     , contains = \ b -> let m = breit b `div` 2 in
	    breit b ==  2*m+1 && hoch b == 2*m+1  && and ( do
		  x <- [ 0 .. breit b - 1 ]
		  y <- [ 0 .. hoch  b - 1 ]
		  return $ if x == m && y == m  
			   then '1' == b `at` (x,y)
			   else True )
     , sample = \ c n -> sequence $ replicate c $ do
	   let m = n `div` 2
	   b <- irgendein (2*m+1, 2*m+1) "01" 
	   return $ tos b [ ((m,m), '1') ]

     , anti_sample = \ c n -> do
	  let g = n `div` 2
          -- kein quadrat
          noquad <- irgendein (n, n+1) "01"
          -- kein ungerades quadrat
	  noung <-  irgendein  (2*g,2*g) "01"
	  -- irgendwelche ungeraden quadrate mit 0 in der mitte
	  cands  <- sequence $ replicate c $ do
	      let u = 2 * g + 1
	      p <- irgendein (u, u) "01"
	      return $ p `to` ((g, g), '1')
	  return $ filter (not . contains center) $ noquad : noung : cands


     }


