module Bild.Checker where

import Bild
import Language

import Random
import Set
import ToDoc
import Monad (guard)

checker :: Language Char (Bild Char)
checker = Language 
     { name = "CHECKER"
     , alphabet     = mkSet "01"
     , definition = text "{ b : Schachbrettmuster mit geraden Seiten }"
     , contains = \ b -> even (breit b) && even (hoch b ) && and ( do
		  x <- [ 0 .. breit b - 1 ]
		  y <- [ 0 .. hoch  b - 1 ]
		  return $ b `at` (x,y) == if odd (x+y) then '1' else '0' )
     , sample = \ c n -> sequence $ replicate c $ do
	   let m = n `div` 2
           x <- randomRIO (1,n `div` 2)
	   f <- randomRIO (False, True)
	   let (b,h) = if f then (2*x, 2*m) else (2*m, 2*x)
	   let p = uniform '0' (b,h)
	   return $ tos p $ do
	       x <- [ 0 .. breit p - 1 ]
	       y <- [ 0 .. hoch  p - 1 ]
	       guard $ odd (x+y)
	       return ((x,y), '1')
     , anti_sample = anti checker
     }

