module Bild.Ops where

import Bild.Type

import Monad
import Random
import List (transpose)

auf   :: MonadPlus m => Bild s -> Bild s -> m ( Bild s )
-- bild a  *auf*  bild b gesetzt (falls es pat)
auf a b 
    | ist_leer a = return b
    | ist_leer b = return a
    | breit a == breit b =
        return ( bild ( inhalt a ++ inhalt b ) ) 
    | otherwise = mzero

the :: Maybe o -> o
the (Just o) = o
the Nothing  = error "Ops.the"

neben ::  MonadPlus m => Bild s -> Bild s -> m ( Bild s )
-- bild a  *links neben*  bild b gesetzt (falls es pat)
neben a b 
    | ist_leer a = return b
    | ist_leer b = return a
    | hoch a == hoch b =
        return ( bild ( zipWith (++) (inhalt a) (inhalt b) ) ) 
    | otherwise = mzero

-------------------------------------------------------------------

data Dir = Horizontal | Vertical | Both
     deriving (Eq, Ord, Show, Read)

takeH :: Int -> Bild s -> Bild s
takeH t p = bild $ do z <- inhalt p; return $ take t z

dropH :: Int -> Bild s -> Bild s
dropH t p = bild $ do z <- inhalt p; return $ drop t z

takeV :: Int -> Bild s -> Bild s
takeV t p = bild $ take t $ inhalt p

dropV :: Int -> Bild s -> Bild s
dropV t p = bild $ drop t $ inhalt p

takeD Horizontal = takeH
takeD Vertical = takeV

dropD Horizontal = dropH
dropD Vertical = dropV

dim Horizontal = breit
dim Vertical = hoch

-------------------------------------------------------------------

glues :: [ Bild Char ] -> Bild Char
glues = foldl glue leer

glue :: Bild Char -> Bild Char -> Bild Char
-- nebeneinander, mit Abstand, passend gemacht
glue links rechts = 
    let hl = hoch links; hr = hoch rechts; h = max hl hr
    in	the $ neben ( breiter 1 (tiefer (h - hl) links) )
		    (            tiefer ( h - hr ) rechts )

tiefer :: Int -> Bild Char -> Bild Char
tiefer d b 
    | d <= 0 = b
    | otherwise = bild $ replicate d ( replicate (breit b) ' ' ) ++ inhalt b

breiter :: Int -> Bild Char -> Bild Char
breiter d b
    | d <= 0 = b
    | otherwise = bild $ do 
	let cs = replicate d ' '
	z <- inhalt b 
	return $ z ++ cs

--------------------------------------------------------------------

rahmen :: s -> Bild s -> Bild s
-- zeichne Umrahmung aus Spezialzeichen
rahmen c p =
    let zeile = replicate (breit p) c
	rl z =  [c] ++ z ++ [c]
    in	bild $ map rl $ [ zeile ] ++ inhalt p ++ [ zeile ]

--------------------------------------------------------------------

somewhere :: Bild s -> IO Position
somewhere b = do
    x <- randomRIO (0, breit b - 1)
    y <- randomRIO (0, hoch  b - 1)
    return (x, y)

drehe :: Bild s -> Bild s
drehe p = bild (transpose (inhalt p))

zeile :: Bild s -> Int -> Bild s
-- extrahieren
zeile p i = bild [ inhalt p !! i ]

unzeile :: Bild s -> Int -> Bild s
-- lschen
unzeile p i = 
    let (pre, z : post) = splitAt i $ inhalt p
    in	bild (pre ++ post)

spalte :: Bild s -> Int -> Bild s
-- extrahieren
spalte p j = bild $ do
       z <- inhalt p
       return [ z !! j ]

unspalte :: Bild s -> Int -> Bild s
-- lschen
unspalte p j = bild $ do
       z <- inhalt p
       let (pre, c : post) = splitAt j z
       return ( pre ++ post )

to :: Bild s -> (Position, s) -> Bild s
-- Buchstaben an Position schreiben
to b ((x,y), s) = 
    let (pre, z : post) = splitAt y $ inhalt b
	(bre, c : bost) = splitAt x $ z
    in	bild ( pre ++ ( bre ++ s : bost ) : post )

tos :: Bild s -> [ (Position, s) ] -> Bild s
tos = foldl to 



