module Bild.Permutation where

import Bild
import Language

import Random
import Set
import ToDoc

someperm :: [a] -> IO [ a ]
-- erzeugt irgendeine (harhar) permutation der eingabeliste
someperm [] = return []
someperm [x] = return [x]
someperm (x : xs) = do
    rest <- someperm xs
    k <- randomRIO (0, length rest)
    let (pre, post) = splitAt k rest
    return $ pre ++ x : post

perm :: Language Char (Bild Char)
perm = Language 
     { name = "PERM"
     , alphabet     = mkSet "01"
     , definition = text "{ b : ist Permutationsmatrix }"
     , contains = \ b -> let zeile_ok xs = length (filter (== '1') xs) == 1
			         && length (filter (== '0') xs) 
				        == length xs - 1
			     zeilen_ok xss = and $ map zeile_ok xss
			 in  breit b == hoch b
		             && zeilen_ok (inhalt b) 
			     && zeilen_ok (inhalt (drehe b))
     , sample = \ c n -> sequence $ replicate c $  do
	   let b = uniform '0' (n, n)
	   let xs = [ 0 .. n - 1 ]
	   ys <- someperm xs
	   return $ tos b [ ((x,y), '1') | (x,y) <- zip xs ys ]
     , anti_sample = anti perm
     }

perm2 :: Language Char (Bild Char)
perm2 = Language 
     { name = "PERM2"
     , alphabet     = mkSet "012"
     , definition = text "{ b .| 2^+- .| b  : ist Permutationsmatrix }"
     , contains = \ b -> let n = hoch b
			     l = takeH n b; r = dropH (n+1) b 
			 in     breit b == 2 * n + 1
			     && spalte b n == uniform '2' (1, n)
			     && contains perm l
			     && l == r
			     
     , sample = \ c n -> do
           bs <- sample perm c n
	   return $ do 
	       b <- bs 
	       let h = hoch b
	       return $ the $ neben b 
		      $ the $ neben (uniform '2' (1,h))
		      $ b

     , anti_sample = \ c n -> do
           bs <- sample perm c n 
	   sequence $ do b <- bs ; return $ do
	           c <- edit b
		   return $ the $ neben b 
		      $ the $ neben (uniform '2' (1,hoch b))
		      $ c
	       
     }



