module Bild.Type

( Bild (), dims
, bild
, irgendein

, breit, hoch, inhalt
, leer, ist_leer
, row, column, uniform

, at
, Position
, Direction (..)
, im_bild
, alpha 
)

where

import Bild.Position

import Language.Object
import Language.Alpha
import Language.Address

import Size
import ToDoc

import Set
import Random
import Monad (guard)
import Maybe

data Bild s = Bild
	  { breit :: Int
	  , hoch :: Int
	  , inhalt :: [ [s] ]
	    -- alle Strings mssen wirklich Lnge `breit haben
	    -- und es mssen wirklich `hoch stck davon sein
	  }
    deriving (Eq, Ord)

---------------------------------------------------------

instance Ord s => Alpha s (Bild s) where
     alpha = mkSet . concat . inhalt

dims :: Bild s -> Position
dims b = (breit b, hoch b)

leer :: Bild s
leer = Bild { breit = 0, hoch = 0, inhalt = [] }

ist_leer :: Bild s -> Bool
ist_leer b = (0 == hoch b) || (0 == breit b)

im_bild :: Bild s -> Position -> Bool
im_bild b (x,y)
    | x < 0       = False
    | x >= breit b = False
    | y < 0	  = False
    | y >= hoch  b = False
    | otherwise = True

bild :: [ [s] ] -> Bild s
bild xss0 =
    let xss = filter (not . null) xss0
	lengths = map length xss
	b = maximum lengths; b' = minimum lengths
	h = length xss
    in	if 0 == h then leer
	else if b /= b'
	     then error "Bild.bild fr verschieden lange Zeilen"
	     else if 0 == b then leer
		  else Bild { breit = b, hoch = h, inhalt = xss }

row :: [s] -> Bild s
row xs = bild [ xs ]

column :: [s] -> Bild s
column xs = bild $ [ [x] | x <- xs ]


uniform :: s -> (Int, Int) -> Bild s
uniform c (b,h) = bild $ replicate h $ replicate b c

----------------------------------------------------------------------

instance Read [s] => Read  ( Bild s ) where
   -- kann nur flache Bilder lesen
   readsPrec p cs = do
       ( s, rest ) <- readsPrec p cs
       return ( row s, rest )

instance Size (Bild s) where
    size b = max (hoch b) (breit b)

instance Ord s => Object s (Bild s) where
    -- perhaps nicht so sinnvoll,
    -- weil alle formate (abmessungen) gemischt werden
    someIO n a = do
	m <- randomRIO (1, n)
        f <- randomRIO (False, True)
	let (b,h) = if f then (n,m) else (m,n)
	irgendein (b,h) (setToList a)

irgendein :: (Int, Int) -> [ s ] -> IO (Bild s)
irgendein (b,h) s = do
	let letter = do k <- randomRIO (0, length s - 1)
			return $ s !! k
	zeilen <- sequence $ replicate h
		$ sequence $ replicate b
		$ letter
	return $ bild zeilen

-------------------------------------------------------------------------

data Direction = North | East | South | West
     deriving ( Eq, Ord, Show, Read )

instance ToDoc Direction where toDoc = text . show

instance Directed Position  Direction where
    move (x,y) d = case d of
	 North -> ( x     , y - 1 )
	 East  -> ( x + 1 , y     )
	 South -> ( x     , y + 1 )
	 West  -> ( x - 1 , y     )

instance Address a (Bild a) Position where
    peek b (x, y) = do
	 guard $ im_bild b (x, y)
	 return $ inhalt b !! y !! x
	 
at :: Bild s -> Position -> s
at b p = fromMaybe ( error $ "Position nicht im Bild: " ++ show p )
       $ peek b p 

instance Top (Bild a) Position where
    top b = (0, 0)

instance Whoami Position where
    whoami p = "" -- unsinnvoll
