module Fourway.Rechnung 

where

import ToDoc

import Bild.Type hiding ( bild )
import Bild.Ops  ( rahmen )

import Fourway.Type
import Konfig
import Fourway.Bewegung

import Alternate
import qualified Alternate.Status as S

import Set
import FiniteMap
import Monad ( guard )


instance FourwayC s z 
	 => Maschine ( Fourway s z ) ( Bild s ) ( Konfig s z )


state :: (Ord z, Show z) 
      => Fourway s z -> z -> S.Status
state m q = 
    case ( elementOf q (existenziell m), elementOf q (universell m) ) of
	 ( True , False ) -> S.Existenziell
	 ( False, True  ) -> S.Universell
	 ( False, False ) -> error $ "Fourway: unbekannter Zustand: " ++ show q
	 ( True , True  ) -> error $ "Fourway: doppelter Zustand: "   ++ show q

instance ( Ord z, Show z )
	 => Start ( Fourway s z ) ( Bild s ) ( Konfig s z ) where
    start m e = Konfig { bild = rahmen (randzeichen m) e
		       , pos = (1,1) -- oben links im Rahmen
		       , zustand = startet m
		       , status = state m (startet m)
		       }

instance FourwayC s z 
	 => Next ( Fourway s z ) ( Konfig s z ) where
    next m k = do
        let q = zustand k ; p = pos k ; b = bild k
	    c = b `at` p
	    r = randzeichen m

	(q', d) <- setToList
		$ lookupWithDefaultFM ( tafel m ) emptySet ( q, c )
	let p' = move p d

	-- unerlaubte Transitionen werden schweigend ignoriert.
	-- es ist auch denkbar, da hier harte Fehlermeldungen erfolgen.

	-- darf nicht aus Bild (d. h. ber Rahmen) fahren
	guard $ im_bild b p'
	-- darf nicht auf Rahmen spazierenfahren
	guard $ not ( ( r == c ) && ( r == ( b `at` p' ) ) )
	        
	return $ Konfig { bild = b -- bleibt gleich
			, pos = p'
			, zustand = q'
			, status = state m q'
			}

