module Machine.Walk where

import Machine.World
import Language.Address

import ToDoc


data Question d = Exists d
		| Whoami
		| Read
     deriving (Eq, Ord, Show, Read)

instance Show d => ToDoc (Question d) where 
    toDoc = text . show


data Action d = Go d
     deriving (Eq, Ord, Show, Read)
instance Show d => ToDoc (Action d) where 
     toDoc = text . show

data State o p = State { contents :: o
		   , position :: p
		   }
     deriving (Eq, Ord, Show, Read)

instance (ToDoc o, ToDoc p, ToDoc a, Address a o p) 
	 => ToDoc (State o p) where
     toDoc s = toDoc (position s) <+> toDoc (peek (contents s) (position s))

instance (Top b c , Address a b c ) 
	 => Initial b ( State b c ) where
    initial o = State { contents = o, position = top o }


instance ( Directed p d , Address a o p, Whoami p
	 , ToDoc (State o p), Show (Question d), Show a )
	 => Ask (State o p) (Question d) where
    ask s (Exists d) = 
        case peek (contents s) $ move (position s) d of
	     Nothing -> "false"; Just _ -> "true"

    ask s Read =
	case peek (contents s) (position s) of
	     Just x -> show x
	     Nothing -> ""
    ask s Whoami = show $ whoami $ position s
	

instance ( Directed p d , Address a o p, ToDoc (State o p), Show (Action d) )
	 => Act (State o p) (Action d) where
    act s (Go d)     = s { position = move ( position s ) d}

