module RX.Generate 

( gen
, generate

-- HIDE
, gen_range, pick

, language
, port, portrait
)

where

import Bild
import RX.Type
import RX.Match
import RX.Dove

import Language
import Size
import ToDoc

import List (nub)
import Random

language :: RX -> Language Char (Bild Char)
language x = l where
   l = Language { name = "reg-exp"
	     , definition = toDoc x
	     , alphabet = alpha x
	     , contains = match x
	     , sample = \ c n ->
	           pick c n $ gen_range x (n, n + c)
	     , anti_sample = anti l		   
	     , _size = size x
	     }


pick :: Int -> Int  -> [a] -> IO [a]
pick 0 w xs = return []
pick c w [] = return []
pick c w xs = do
      let (pre, post) = splitAt w xs
      i <- randomRIO (0, length pre - 1)
      rest <- pick (c - 1) (w + 0) $ post
      return $ xs !! i  :  rest
				

---------------------------------------------------------------

gen :: RX -> [ Bild Char ]
-- willkrliche kleine zahlen
gen x = take 20 $ nub $ generate x (0,0) ++ gen_range x (1, 10)

gen_range :: RX -> (Int, Int) -> [ Bild Char ]
gen_range x (lo, hi) = dovetail $ do
    s <- [ lo .. hi ]
    return $ take 5 $  dovetail $ do
        b <- [ 1 .. s - 1 ]
	return $ generate x (b, s - b)

-- nur Bilder fixierte Breite/Hhe
type Constraint = (Int, Int)


fulfill :: Constraint -> Bild s -> Bool
fulfill (b, h) p
    | ist_leer p = 0 == b || 0 == h
    | otherwise  = b == breit p && h == hoch p

generate :: RX -> Constraint -> [ Bild Char ]

generate (Letter c) con =
    let p = bild [[ c ]] 
    in	[ p | fulfill con p ]

generate (Union l r) con =
    merge ( generate l con ) ( generate r con )
generate (Intersection l r) con = 
    [ p | p <- generate l con, match r p ]
generate (Difference l r) con = 
    [ p | p <- generate l con, not $ match r p ]

-----------------------------------------------------------------

generate (Dot Horizontal l r) con = helperH 0 l r con
generate (Dot Vertical   l r) con = helperV 0 l r con

generate (Star d x) con 
    =  [ leer | fulfill con leer ] ++ generate (Plus d x) con

generate e @ (Plus Vertical x) con 
    =  generate x con ++ helperV 1 x e con

generate e @ (Plus Horizontal x) con 
    =  generate x con ++ helperH 1 x e con

generate (Plus Both x) con 
    = generate (Plus Vertical (Plus Horizontal x)) con




helperH start l r (b, h) =  nub $ do
    bl <- [ start .. b - start ]
    x <- generate l ( bl, h )
    y <- generate r ( b - breit x, h )
    x `neben` y

helperV start l r (b, h) =  nub $ do
    hl <- [ start .. h - start ]
    x <- generate l ( b, hl )
    y <- generate r ( b, h - hoch x )
    x `auf` y







