module CFS where

import Grammatik
import Schichten
import Set
import Splits
import Monad (guard)

import ToDoc
import Right
import Wrong


type RS = [(String, String)]

next :: RS -> String -> Set String
next rules xs = mkSet $ do
    (links, x) <- splits xs
    (lhs, rhs) <- rules
    let (mitte, rechts) = splitAt (length lhs) x
    guard $ mitte == lhs
    return $ links ++ rhs ++ rechts

nachfolger :: RS -> String -> [ String ]
nachfolger rs start = do
    ws <- schichten (next rs) start
    setToList ws


g1 :: Grammatik
g1 = Grammatik
   { terminale = mkSet "01#"
   , nichtterminale = mkSet "S"
   , startsymbol = 'S'
   , regeln = mkSet [ ("S", rhs) | rhs <- [ "#", "0S1", "01S0", "1S101" ] ]
   }

g2 :: Grammatik
g2 = Grammatik
   { terminale = mkSet "01#"
   , nichtterminale = mkSet "T"
   , startsymbol = 'T'
   , regeln = mkSet [ ("T", rhs) | rhs <- [ "1#1", "0T0", "1T1" ] ]
   }
    
bewerte :: String -> String -> IO String
bewerte mat w = do
    let n = length w
    let msg = "OK # Size: " ++ show n
    putStrLn $ unlines
        [ "Sie behaupten, das Wort"
	, "w = " ++ render (toDoc w)
	, "der Lnge " ++ show n
	, "wird von beiden Grammatiken erzeugt."
	]
    fs <- sequence [ handle g w | g <- [ g1, g2 ] ]
    if and fs 
       then right_with msg
       else wrong


distill :: Grammatik -> RS
-- Regeln rumdrehen und als rewrite-system auffassen
-- ist bei obigen grammatiken effizient
distill g = do
    (u, v) <- setToList $ regeln g
    return (v, u)

handle :: Grammatik -> String -> IO Bool
handle g w = do
    let r = distill g
    let cut = 1000
    let vs = take cut $ nachfolger r w
    let v = last vs
    let f = length vs < cut && v == [ startsymbol g ]
    putStrLn $ unlines
	     [ "Die Grammatik  G  ist"
	     , render $ toDoc g
	     , "Sie erzeugt das Wort  w ? ... " ++ show f
	     ]
    return f
    



